<?php
class AccountingReportListasRepository extends EntityRepository {
    
    public function getAccountsPayable(){
        $query = "SELECT "
                . "id_vendor,"
                . "fxGetVendorName(id_vendor)as proveedorName,"
                . "COUNT(id)as cantidad_facturas,"
                . "SUM(total - payments)as total "
                . "FROM purchases "
                . "WHERE status = '1' "
                . "AND (total - payments) > 0 "
                . "GROUP BY id_vendor ";
        
        $result = $this->query($query);
        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        return null;
    }
    
    public function getAccountsPayableDetails($options){
        $vendor_type = null;
        
        if($options){
            if(isset($options['vendor_type'])){$vendor_type = " AND pr.type = '{$options['vendor_type']}'";}
        }
        $query = "SELECT "
                . "IF(due_date < CURDATE(),1,0)as vencido, "
                . "s.id_vendor as vendor_id,"
                . "fxGetVendorName(s.id_vendor)as proveedor,"
                . "s.id as compra,"
                . "s.reference as referencia,"
                . "DATE_FORMAT(s.date,'%d/%m/%Y')as fecha,"
                . "DATE_FORMAT(s.due_date,'%d/%m/%Y')as fecha_pago,"
                . "s.total as importe,"
                . "SUM( CASE WHEN p.forma_de_pago != 8 THEN IFNULL(p.monto,0) ELSE 0 END)as pagos_abonos,"
                . "SUM( CASE WHEN p.forma_de_pago = 8 THEN IFNULL(p.monto,0) ELSE 0 END) as nota_credito,"
                . "s.total - SUM(IFNULL(p.monto,0))  as saldo  "
                . "FROM  purchases s "
                . "LEFT JOIN pagos_detalle d ON d.id_factura = s.id "
                . "LEFT JOIN pagos p ON p.id = d.id_pago "
                . "LEFT JOIN vendors pr ON s.id_vendor = pr.id "
                . "WHERE s.status = 1 "
                . "$vendor_type "
                . "GROUP BY s.id "
                . "HAVING saldo > 0 "
                . "ORDER BY fxGetVendorName(s.id_vendor) ASC,s.date ASC ";
        
        $result = $this->query($query);
        if($result->num_rows > 0){
            return array('data'=>$this->resultToArray($result));
        }
        
        return null;
    }
    
    public function getVendorStatement(array $options){
        $vendor = null;
        if ($options['vendor'] != null) {
            $vendor = implode(',', $options['vendor']);
        }

        $fecha = $this->createFilterFecha($options['startDate'], $options['endDate'], "v.date");
        $vendor = $options['vendor'] ? " AND find_in_set(v.vendor,'$vendor') " : null;
        $balanceCondition = isset($options['pending_balance_only']) ? " AND (v.total - v.payments) > 0 " : null;

        $query = "SELECT 
                    c.id as vendor_id, 
                    DATE_FORMAT(v.date,'%m/%d/%Y')as date,
                    c.name,
                    c.contact as contact_name,
                    c.phone,
                    c.email,                              
                    v.id as id_factura, 
                    v.reference,
                    v.total,
                    (v.total - v.payments) as balance,
                    fxGetStatusName(v.`status`,'status','StoreIn')as statusName
                    from purchases v 
                    join vendors as c on v.vendor = c.id                     
                    WHERE v.status != 3
                    $fecha
                    $vendor $balanceCondition";


        $result = $this->query($query);
        if ($result->num_rows > 0) {
            $result = $this->resultToArray($result);
            $estadoDeCuena = array();
            $repoPagos = new PagoRepository();

            foreach ($result as $row) {
                $datosCliente = array(
                    'name' => $row['name'],
                    'contact_name' => $row['contact_name'],
                    'phone' => $row['phone'],
                    'email' => $row['email'],
                );
                //unset($row['nombre'],$row['rfc'],$row['contacto'],$row['telefono'],$row['email']);

                $pagos = $repoPagos->getPagosByIdFactura($row['id_factura']);
                $row['pagos'] = $pagos;
                $estadoDeCuena[$row['vendor_id']]['datosCliente'] = $datosCliente;
                $estadoDeCuena[$row['vendor_id']]['facturas'][$row['id_factura']] = $row;
            }

            return array(
                'data' => $estadoDeCuena,
            );
        } else {
            return null;
        }
        return null;
    }
    
    public function getInventoryCost($options){
        $inventorySupplieRepo = new InventorySupplieRepository();
        $tools = new Tools();
        $requiredMonth = $tools->setFormatDateToDB($options['startDate']);
        
        $dateTime1 = new DateTime($requiredMonth);
        $dateTime2 = new DateTime(date('Y-m-d'));
       
        if ($dateTime1->format('Y-m') === $dateTime2->format('Y-m')) {
            $inventorySupplieRepo->calculateAverageCost();
            
            $subtitle = " Al ".$dateTime1->format('d/m/Y');
            $query = "SELECT 
                        p.code as code,                        
                        p.accounting_account,
                        p.description as description,
                        fxGetPresentacionDescripcion(p.presentation_id)as presentation,
                        fxGetMarcaDescripcion(p.brand_id)as brand,
                        fxGetUMDescription(p.um_id)as unit_measurement,     
                        
                        SUM(IFNULL(i.quantity,0))as final_inventory,
                        IFNULL(i.average_cost,0) as final_average_cost,
                        SUM(IFNULL(m.quantity,0)) as initial_inventory,
                        IFNULL(m.average_cost,0) as initial_average_cost,
                        SUM(IFNULL(i.quantity,0))as current_inventory,
                        IFNULL(i.average_cost,0) as current_average_cost
                       
                    FROM supplies p 
                    LEFT JOIN month_end_inventory m ON m.idProduct = p.id AND (m.month = '{$requiredMonth}' OR m.month = SUBDATE(LAST_DAY(CURDATE()), INTERVAL 1 MONTH) )
                    LEFT JOIN inventory_supplies i ON i.idProduct = p.id
                    WHERE p.type = 'insumo'
                    
                    GROUP BY p.id
                    ORDER BY p.description ASC";
            
            $result = $this->query($query);
        }else{
            $subtitle = " Al ".$dateTime1->format('t/m/Y');
            $lastMonth = date($requiredMonth, strtotime("last day of previous month"));
            $query = "SELECT 
                        p.code as code,
                        p.accounting_account,
                        p.description as description,
                        fxGetPresentationDescription(p.presentation)as presentation,
                        fxGetBrandDescription(p.brand)as brand,
                        fxGetUMDescription(p.unit_of_measurement)as unit_measurement,                        
                        SUM(CASE WHEN i.month = '{$lastMonth}' THEN IFNULL(i.quantity,0) ELSE 0 END )as initial_inventory,                            
                        (CASE WHEN i.month = '{$lastMonth}' THEN IFNULL(i.average_cost,0) ELSE 0 END )as initial_average_cost,
                        SUM(CASE WHEN i.month = '{$requiredMonth}' THEN IFNULL(i.quantity,0) ELSE 0 END )as final_inventory,
                        (CASE WHEN i.month = '{$requiredMonth}' THEN IFNULL(i.average_cost,0) ELSE 0 END )as final_average_cost,
                        SUM(IFNULL(ci.quantity,0))as current_inventory,
                        IFNULL(ci.average_cost,0) as current_average_cost
                        
                    FROM supplies p 
                    LEFT JOIN month_end_inventory i ON i.idProduct = p.id AND (i.month = '{$requiredMonth}' OR i.month = '$lastMonth')
                    LEFT JOIN inventory_supplies ci ON ci.idProduct = p.id
                    WHERE p.type = 'insumo'
                    GROUP BY p.id
                    ORDER BY p.description ASC";
            
            $result = $this->query($query);            
        }   
            
        if($result->num_rows > 0){
            $inventory = $this->resultToArray($result);
            
            $array = array();
            foreach($inventory as $row){
                $row['initial_inventory_cost'] = $row['initial_inventory'] * $row['initial_average_cost'];
                $row['final_inventory_cost'] = $row['final_inventory'] * $row['final_average_cost'];
                $row['current_inventory_cost'] = $row['current_inventory'] * $row['current_average_cost'];
                $array[] = $row;
            }
            
            return array(
                'data'=>$array,
                'subtitle'=>$subtitle
            );
        }

        return null;
    }
        
     public function createFilterFecha($fechaInicio,$fechaFin,$campoFecha = null ){
        $fecha = null;
        $tools = new Tools();
        if($fechaInicio!=null){
            $fechaInicio = $tools->setFormatDateToDB($fechaInicio);
            if($fechaFin!=null){
                $fechaFin = $tools->setFormatDateToDB($fechaFin);
                $fecha .=" AND $campoFecha BETWEEN '{$fechaInicio}' AND '{$fechaFin}' ";
            }else{
                $fecha .=" AND $campoFecha BETWEEN '{$fechaInicio}' AND '{$fechaInicio}' ";
            }
        }elseif($fechaFin!=null){
            $fecha .=" AND $campoFecha BETWEEN '{$fechaFin}' AND '{$fechaFin}' ";
        }
        
        return $fecha;
    }
}