<?php

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

class AppointmentRepository extends EntityRepository {

    private $table = 'appointments';
    public $flashmessenger = null;
    private $options = array(
        'appointment_num'=>null,
        'appointment_date'=>null,
        'appointment_hour'=>null,
        'customer_id'=>null,
        'sales_person'=>null,
        'event_type_id'=>null,
        'status'=>null
    );
    
    public function __construct() {
        if(!$this->flashmessenger instanceof FlashMessenger){
            $this->flashmessenger = new FlashMessenger();
        }
    }
    
    public function setOptions($data){
      foreach ($this->options as $option => $value){
          if(isset($data[$option])){
            $this->options[$option] = $data[$option];
          }
      }
    }
  
    public function getOptions(){
        return $this->options;
    }
    
    public function getTable(){
        return $this->table;
    }

    public function save(array $data, $table = null) {
        $login = new Login();
        $tools = new Tools();
        $data['appointment_date'] = $tools->setFormatDateToDB($data['appointment_date']);
        $data['sales_person'] = $login->getId();
        $data['appointment_num'] = $this->getNextAppointmentNumber();
        $data['status'] = 1;

        $rs = parent::save($data, $this->table);
        
        if($rs){
            $ajax = new AppointmentAjax();
            $ajax->mailingAppointment(array('appointment_id'=>$this->getInsertId()));
        }
        
        return $rs;
    }
    
    public function delete($id, $table = null) {        
        return parent::delete($id, $this->table);
    }

    public function update($id, $data, $table = null) {
        $tools = new Tools();
        $data['appointment_date'] = $tools->setFormatDateToDB($data['appointment_date']);
        unset($data['appointment_num']);
        
        return parent::update($id, $data, $this->table);
    }

    public function getById($id, $table = null,$selectAux = null) {
        $select = "SELECT a.*,"
                . "DATE_FORMAT(a.appointment_date,'%m/%d/%Y')as formated_date,"
                . "TIME_FORMAT(a.appointment_hour,'%h:%s %p')as formated_hour,"
                . "TIME_FORMAT(a.appointment_hour,'%H:%s')as appointment_hour,"
                . "c.nombre as customer_name,"
                . "c.telefono as customer_phone,"
                . "c.correo as customer_email,"
                . "e.name as event_type_name, "
                . "fxGetUserName(a.creado_por)as sales_person_name, "
                . "fxGetStatusName(a.status,'status','Appointment')as status_name "
                . "FROM $this->table a "
                . "LEFT JOIN clientes c ON a.customer_id = c.id "
                . "LEFT JOIN appointment_events_type e ON a.event_type_id = e.id "
                . "WHERE a.id = '$id' "
                . "LIMIT 1";
        $result = $this->query($select);

        if ($result->num_rows > 0) {
           return $this->resultToArray($result)[0];
        }
        return null;
    }

    public function isUsedInRecord($id, array $buscarEn = null,$andWhere = null) {
        return parent::isUsedInRecord($id, array('productos' => 'marca'));
    }

    public function getListaSelectAppointments() {
        $select = "SELECT * FROM $this->table ";
        $result = $this->query($select);

        if ($result) {
            $array = array();
            while ($row = $result->fetch_assoc()) {
                $array[$row['id']] = $row['nombre'];
            }
            return $array;
        }
        return null;
    }
    
     public function getListaAppointments() {
        $select = "SELECT *,"
                . "DATE_FORMAT(appointment_date,'%m/%d/%Y')as formated_date,"
                . "TIME_FORMAT(appointment_hour,'%h:%s %p')as formated_hour,"
                . "fxGetClienteName(customer_id)as customer_name,"
                . "fxGetUserName(creado_por)as sales_person_name, "
                . "fxGetStatusName(status,'status','Appointment')as status_name "
                . "FROM $this->table "
                . "ORDER BY id DESC";
        $result = $this->query($select);

        if ($result) {
           return $this->resultToArray($result);
        }
        return null;
    }
    
     public function getTypeEvents() {
        $select = "SELECT * FROM appointment_events_type ";
        $result = $this->query($select);

        if ($result) {
            $array = array();
            while ($row = $result->fetch_assoc()) {
                $array[$row['id']] = $row['name'];
            }
            return $array;
        }
        return null;
    }
    
    public function getAppointmentsByDate($date,$id = null){
        $tools = new Tools();
        $date = $tools->setFormatDateToDB($date);
        
        if($id){$id = " AND id != '$id' ";}
        
        $query = "SELECT appointment_hour FROM $this->table WHERE appointment_date = '$date' AND status != '2' $id";
        $result = $this->query($query);
        
        if($result->num_rows >0){
            return $result = $this->query($query);            
        }
        
        return null;
    }
    
    public function getListStatus(){
        $query = "SELECT * FROM codigo_status WHERE tipo = 'Appointment'";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $array = array();
            foreach($result as $status){
                $array[$status['codigo']] = $status['descripcion'];
            }
            return $array;
        }
        return null;
    }
    
    public function getNextAppointmentNumber(){
        $query = "SELECT appointment_num FROM $this->table ORDER BY appointment_num DESC LIMIT 1 ";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $result = $result->fetch_object();
            $appointmentNumber = $result->appointment_num;
            $appointmentNumber++;
            return $appointmentNumber;
        }
        return 1;
    }
}