<?php

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

class NotificationRepository extends EntityRepository {

    private $table = 'notifications';
    public $flashmessenger = null;
    private $options = array(
        'notification_num'=>null,
        'subject'=>null,
        'message'=>null,
        'send_to'=>null,
        'when_send'=>null,
        'repeat'=>0,
        'repeat_from'=>null,
        'repeat_to'=>null,
        'repeat_hour'=>null,
        'status'=>null
    );
    
     private $options_aux = array(
        'send_now'=>null
    );
    
    public function __construct() {
        if(!$this->flashmessenger instanceof FlashMessenger){
            $this->flashmessenger = new FlashMessenger();
        }
    }
    
    public function setOptions($data){
      foreach ($this->options as $option => $value){
          if(isset($data[$option])){
            $this->options[$option] = $data[$option];
          }
      }
      
      foreach ($this->options_aux as $option => $value){
          if(isset($data[$option])){
            $this->options_aux[$option] = $data[$option];
          }
      }      

    }
  
    public function getOptions(){
        return $this->options;
    }
    
    public function getTable(){
        return $this->table;
    }

    public function save(array $data, $table = null) {      
        $tools = new Tools();
        $data['notification_num'] = $this->getNextNotificationNumber();
        $data['when_send'] = $tools->setFormatDateTimeToDB($data['when_send']);
        
        if(is_array($data['send_to']) && count($data['send_to']) > 0){
            $sendTo = implode(',', $data['send_to']);
            $data['send_to'] = trim($sendTo,',');
        }        
        
        if($data['repeat'] == 0){
            unset($data['repeat_from'],$data['repeat_to'],$data['repeat_hour']);
        }else{
            $data['repeat_from'] = $tools->setFormatDateToDB($data['repeat_from']);
            $data['repeat_to'] = $tools->setFormatDateToDB($data['repeat_to']);
            $data['repeat_hour'] = $tools->setTimeToDB($data['repeat_hour']);
        }              
        
        if($this->options_aux['send_now']){
            $data['when_send'] = date('Y-m-d H:i:s');
            $rs = parent::save($data, $this->table);        
            
            if($rs){
                if($this->sendNotification(parent::getInsertId())){
                    $this->flashmessenger->addMessage(array('success'=>'Mensaje enviado correctamente.'));
                }else{
                    $this->flashmessenger->addMessage(array('success'=>'Error al tratar de enviar mensaje.</br> Intente usando el boton <strong>Enviar este mensaja ahora.</strong>'));
                }
                return true;
            }else{
                return null;
            }            
        }        
        
        return parent::save($data, $this->table);
    }
    
    public function delete($id, $table = null) {        
        $currentData = $this->getById($id);
        if($currentData['status'] === '3'){return true;}
        
        return parent::update($id, array('status'=>'3'), $this->table);       
       
        //return parent::delete($id, $this->table);
    }

    public function update($id, $data, $table = null) {
        $tools = new Tools();
        $data['when_send'] = $tools->setFormatDateTimeToDB($data['when_send']);
        
        if(is_array($data['send_to']) && count($data['send_to']) > 0){
            $sendTo = implode(',', $data['send_to']);
            $data['send_to'] = trim($sendTo,',');
        }        
        
        if($data['repeat'] == 0){
            $data['repeat_from'] = "_NULL";
            $data['repeat_to'] = "_NULL";
            $data['repeat_hour'] = "_NULL";
        }else{
            $data['repeat_from'] = $tools->setFormatDateToDB($data['repeat_from']);
            $data['repeat_to'] = $tools->setFormatDateToDB($data['repeat_to']);
            $data['repeat_hour'] = $tools->setTimeToDB($data['repeat_hour']);
        }      
        
        unset($data['notification_num']);
        
        return parent::update($id, $data, $this->table);
    }

    public function getById($id, $table = null,$selectAux = null) {
        $select = "SELECT a.*,"
                . "DATE_FORMAT(a.when_send,'%m/%d/%Y %h:%i %p')as when_send,"
                . "DATE_FORMAT(a.repeat_from,'%m/%d/%Y')as repeat_from,"
                . "DATE_FORMAT(a.repeat_to,'%m/%d/%Y')as repeat_to,"
                . "TIME_FORMAT(a.repeat_hour,'%h:%i %p')as repeat_hour,"
                . "fxGetUserName(a.creado_por)as created_by_name, "
                . "DATE_FORMAT(a.creado_fecha,'%m/%d/%Y %h:%i %p')as created_date,"
                . "fxGetStatusName(a.status,'status','Notification')as status_name "
                . "FROM $this->table a "
                . "WHERE a.id = '$id' "
                . "LIMIT 1";
        $result = $this->query($select);

        if ($result->num_rows > 0) {
           return $this->resultToArray($result)[0];
        }
        return null;
    }

    public function isUsedInRecord($id, array $buscarEn = null,$andWhere = null) {
        return parent::isUsedInRecord($id, array('productos' => 'marca'));
    }
    
     public function getListaNotifications() {
        $select = "SELECT n.*,"
                . "DATE_FORMAT(n.when_send,'%m/%d/%Y %h:%i %p')as when_send,"
                . "DATE_FORMAT(n.repeat_from,'%m/%d/%Y')as repeat_from,"
                . "DATE_FORMAT(n.repeat_to,'%m/%d/%Y')as repeat_to,"
                . "TIME_FORMAT(n.repeat_hour,'%h:%i %p')as repeat_hour,"
                . "fxGetUserName(n.creado_por)as created_by_name, "
                . "DATE_FORMAT(n.creado_fecha,'%m/%d/%Y %h:%i %p')as created_date,"
                . "fxGetStatusName(n.status,'status','Notification')as status_name,"
                . "DATE_FORMAT(MAX(sent_date),'%m/%d/%Y %h:%i %p')as last_sent_date "
                . "FROM $this->table n "
                . "LEFT JOIN notifications_sent s ON s.notification_id = n.id "
                . "GROUP BY n.id "
                . "ORDER BY n.id DESC";
        $result = $this->query($select);

        if ($result) {
           return $this->resultToArray($result);
        }
        return null;
    }
    
    public function getListStatus(){
        $query = "SELECT * FROM codigo_status WHERE tipo = 'Notification'";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $array = array();
            foreach($result as $status){
                $array[$status['codigo']] = $status['descripcion'];
            }
            return $array;
        }
        return null;
    }
    
    public function getNextNotificationNumber(){
        $query = "SELECT notification_num FROM $this->table ORDER BY notification_num DESC LIMIT 1 ";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $result = $result->fetch_object();
            $notificationNumber = $result->notification_num;
            $notificationNumber++;
            return $notificationNumber;
        }
        return 1;
    }
    
    public function sendtoStringToArray($sendTo){
        $array = array();
        $sendToArray = explode(',', $sendTo);
        foreach($sendToArray as $key => $user){
            $array[$user] = $user;
        }
        return $array;
    }
    
    public function sendNotification($id){      
        $query = "SELECT * FROM $this->table WHERE id = '$id'";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $notification = $this->resultToArray($result)[0];
            
            if($notification['send_to'] == '0'){// 0 = Todos los usuarios
                $userRepo = new UsuarioRepository();
                $users = $userRepo->getListaSelectUsuarios();
            }else{
                $usersArray = explode(',', $notification['send_to']);  
                foreach($usersArray as $user => $userId){
                    $users[$userId] = $userId;
                }                    
            }              
            
            foreach($users as $user => $userId){
                $key_notification = str_replace(' ', '', $notification['id'].'_'.$userId.'_'.$notification['when_send']);
                $sent_date = date('Y-m-d H:i:s');

                $array = array(
                    'notification_id'=>$notification['id'],
                    'user_id'=>$notification['send_to'],
                    'subject'=>$notification['subject'],
                    'message'=>$notification['message'],
                    'sent_date'=>$sent_date,
                    'key_notification'=>$key_notification
                );

                parent::save($array, 'notifications_sent');
            }
            
           return true;
        }       
    }
    
    public function sendScheduledNotifications(){
        /*Traer las notificaciones que se deben enviar hoy*/
        $todayNotifications = array();
        $today = date('Y-m-d');
        $query = "SELECT id,"
                . "subject,"
                . "message,"
                . "send_to,"                
                . "IF(DATE(when_send) != '$today',CONCAT('$today ',repeat_hour),when_send)as when_send,"
                . "TIME_TO_SEC(IF(`repeat` = 1,repeat_hour,TIME(when_send)))as hour_to_send "
                . "FROM notifications "
                . "WHERE status = 1 "
                . "AND DATE(when_send) = '$today' "
                . "OR (`repeat` = 1 AND '$today' BETWEEN repeat_from AND repeat_to)";
      
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $todayNotifications = $this->resultToArray($result);
            
            /*Obtener las notificaciones que ya se enviaron*/
            $sentNotifications = array();
            $sentQuery = "SELECT * FROM notifications_sent WHERE DATE(sent_date) = '{$today}'";
            $rsSentQuery = $this->query($sentQuery);

            if($rsSentQuery->num_rows > 0){
                $rsSentQuery = $this->resultToArray($rsSentQuery);
                foreach($rsSentQuery as $notification){
                    $sentNotifications[$notification['key_notification']] = $notification;
                }
            }
            
            /*Meter en array notificaciones de hoy que no se han enviado*/
            $usersRepo = new UsuarioRepository();
            $tools = new Tools();
            $pendingToSend = array();
            $time = date('H:i:s');
            //$timeToSec = $tools->timeToSec($time);
            
            foreach($todayNotifications as $notification){      
                $users = array();
                /*Asignamos a $users, usuarios a los que se les enviara la notificacion*/
                if($notification['send_to'] == '0'){// 0 = Todos los usuarios
                    $users = $usersRepo->getListaSelectUsuarios();
                }else{
                    $usersArray = explode(',', $notification['send_to']);  
                    foreach($usersArray as $user => $userId){
                        $users[$userId] = $userId;
                    }                    
                }              
               
                /*Agregamos a $pendingToSend notificaciones de cada usuario que se van a enviar*/
                /*Primero validamos sino se ha enviaro, si ya existe en $sentNotifications, ya no se envia.*/
                foreach($users as $userId => $value){
                    $key_notification = str_replace(' ', '', $notification['id'].'_'.$userId.'_'.$notification['when_send']);
                    if(!key_exists($key_notification, $sentNotifications)){
                        $notification['send_to'] = $userId;
                        $pendingToSend[$key_notification] = $notification; 
                    }
                }         
            }            
           
            /*Enviamos notificaciones*/
            $sent_date = date('Y-m-d H:i:s');
            foreach($pendingToSend as $key_notification => $notification){
                $array = array(
                    'notification_id'=>$notification['id'],
                    'user_id'=>$notification['send_to'],
                    'subject'=>$notification['subject'],
                    'message'=>$notification['message'],
                    'sent_date'=>$sent_date,
                    'key_notification'=>$key_notification
                );
                
                parent::save($array, 'notifications_sent');
            }
        }             
    }   
    
    public function getMessagesUnreadByUserId($userId){
        $query = "SELECT * FROM notifications_sent WHERE user_id = '$userId' AND reading_date IS NULL";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        
        return null;
    }
    
    public function getUserReadMessageById($meesageId){
        $query = "SELECT "
                . "fxGetUserName(user_id)as user_name,"
                . "DATE_FORMAT(sent_date,'%m/%d/%Y %h:%i %p')as formated_sent_date, "
                . "DATE_FORMAT(reading_date,'%m/%d/%Y %h:%i %p')as formated_reading_date "
                . "FROM notifications_sent n "
                . "WHERE n.notification_id = '$meesageId'"
                . "AND reading_date IS NOT NULL ";
        
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        
        return null;
    }
    
    public function getUserResponsesMessageById($meesageId){
        /*response_from_owner = 1 significa que son respuestas del admin que envio el mensaje, y agregar una respueata a una respuesta de otro usuario*/
        $query = "SELECT "
                . "notification_sent_id,"
                . "COUNT(n.id)as messages_qty,"
                . "fxGetUserName(response_by)as user_name,"
                . "DATE_FORMAT(sent_date,'%m/%d/%Y %h:%i %p')as formated_sent_date, "
                . "DATE_FORMAT(MAX(response_date),'%m/%d/%Y %h:%i %p')as formated_last_response_date "
                . "FROM notifications_response n "
                . "LEFT JOIN notifications_sent ns ON ns.id = n.notification_sent_id "
                . "WHERE n.notification_id = '$meesageId' "
                . "AND response_from_owner is null "
                . "GROUP BY notification_sent_id ";
        
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        
        return null;
    }
    
    public function getNotificationsPendingToShowOnPopUp($idUser){
        $query = "SELECT * FROM notifications_sent WHERE user_id = '$idUser' AND status_popup_notification = 0";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        
        return null;
    }
    
    public function setNotificationAsShowOnPopup(){
        $query = "UPDATE notifications_sent SET status_popup_notification = 1 ";
        return $this->query($query);
    }
}