<?php

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

class PreorderRepository extends EntityRepository {

    private $table = 'preorders';
    private $options = array(
        'preorder_num'=>null,
        'customer_id'=>null,
        'delivery_date'=>null,
        'event_type_id'=>null,
        'estimated_price'=>null,
        'comments_customer'=>null,
        'comments_sales_person'=>null,
        'comments_reject'=>null,
        'sales_person'=>null,
        'status'=>null,
        'reason_reject'=>null
    );
    
    public function setOptions($data){
      foreach ($this->options as $option => $value){
          if(isset($data[$option])){
            $this->options[$option] = $data[$option];
          }
      }
    }
  
    public function getOptions(){
        return $this->options;
    }
    
    public function getTable(){
        return $this->table;
    }

    public function save(array $data, $table = null) {
        $login = new Login();
        $tools = new Tools();
        $data['delivery_date'] = $tools->setFormatDateTimeToDB($data['delivery_date']);
        $data['sales_person'] = $login->getId();
        $data['preorder_num'] = $this->getNextPreorderNumber();
        
        if($data['reason_reject'] == ''){$data['reason_reject'] = 0;}

        $result = parent::save($data, $this->table);
        
        if($result){$this->setLastInsertId($this->getInsertId());}
        
        return $result;
    }
    
    public function delete($id, $table = null) {        
        return parent::delete($id, $this->table);
    }

    public function update($id, $data, $table = null) {
        $tools = new Tools();
        $data['delivery_date'] = $tools->setFormatDateTimeToDB($data['delivery_date']);
        
        if($data['reason_reject'] == ''){$data['reason_reject'] = 0;}
        
        unset($data['preorder_num']);
        return parent::update($id, $data, $this->table);
    }

    public function getById($id, $table = null,$selectAux = null) {
        $select = "SELECT *,"
                . "DATE_FORMAT(delivery_date,'%m/%d/%Y %h:%s %p')as formated_delivery_date,"
                . "fxGetClienteName(customer_id)as customer_name,"
                . "fxGetUserName(creado_por)as sales_person_name, "
                . "fxGetStatusName(status,'status','Preorder')as status_name "
                . "FROM $this->table "
                . "WHERE id = '$id' "
                . "LIMIT 1";
        $result = $this->query($select);

        if ($result) {
           return $this->resultToArray($result)[0];
        }
        return null;
    }

    public function isUsedInRecord($id, array $buscarEn = null,$andWhere = null) {
        return parent::isUsedInRecord($id, array('productos' => 'marca'));
    }
    
    public function getListaPreorders() {
        $select = "SELECT p.*,"
                . "DATE_FORMAT(p.delivery_date,'%m/%d/%Y %h:%s %p')as formated_delivery_date,"
                . "fxGetClienteName(p.customer_id)as customer_name,"
                . "a.name as event_type_name,"
                . "fxGetUserName(p.creado_por)as sales_person_name, "
                . "fxGetStatusName(p.status,'status','Preorder')as status_name "
                . "FROM $this->table p "
                . "LEFT JOIN appointment_events_type a ON p.event_type_id = a.id "
                . "ORDER BY id DESC";
        $result = $this->query($select);

        if ($result) {
           return $this->resultToArray($result);
        }
        return null;
    }
    
     public function getTypeEvents() {
        $select = "SELECT * FROM appointment_events_type ";
        $result = $this->query($select);

        if ($result) {
            $array = array();
            while ($row = $result->fetch_assoc()) {
                $array[$row['id']] = $row['name'];
            }
            return $array;
        }
        return null;
    }
    
    public function getReasonsReject() {
        $select = "SELECT * FROM preorder_reason_reject ";
        $result = $this->query($select);

        if ($result) {
            $array = array();
            while ($row = $result->fetch_assoc()) {
                $array[$row['id']] = $row['reason'];
            }
            return $array;
        }
        return null;
    }
    
    public function getListStatus(){
        $query = "SELECT * FROM codigo_status WHERE tipo = 'Preorder'";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $array = array();
            foreach($result as $status){
                $array[$status['codigo']] = $status['descripcion'];
            }
            return $array;
        }
        return null;
    }
    
     public function getNextPreorderNumber(){
        $query = "SELECT preorder_num FROM $this->table ORDER BY preorder_num DESC LIMIT 1 ";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $result = $result->fetch_object();
            $preorderNumber = $result->preorder_num;
            $preorderNumber++;
            return $preorderNumber;
        }
        return 1;
    }
}