<?php

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

class PromotionRepository extends EntityRepository {

    private $table = 'promotions';
    public $flashmessenger = null;
    private $options = array(
        'tipo'=>null,
        'name'=>null,
        'code'=>null,
        'percentage'=>null,       
        'start_date'=>null,
        'end_date'=>null,
        'status'=>null
    );
    
    public function __construct() {
        if(!$this->flashmessenger instanceof FlashMessenger){
            $this->flashmessenger = new FlashMessenger();
        }
    }
    
    public function setOptions($data){
      foreach ($this->options as $option => $value){
          if(isset($data[$option])){
            $this->options[$option] = $data[$option];
          }
      }
    }
  
    public function getOptions(){
        return $this->options;
    }
    
    public function getTable(){
        return $this->table;
    }

    public function save(array $data, $table = null) {        
        $tools = new Tools();
        $data['promotion_num'] = $this->getNextPromotionNumber();
        $data['start_date'] = $tools->setFormatDateTimeToDB($data['start_date']);
        $data['end_date'] = $tools->setFormatDateTimeToDB($data['end_date']);

        return parent::save($data, $this->table);     
    }
    
    public function delete($id, $table = null) {        
        return parent::delete($id, $this->table);
    }

    public function update($id, $data, $table = null) {
        $tools = new Tools();
        $data['start_date'] = $tools->setFormatDateTimeToDB($data['start_date']);
        $data['end_date'] = $tools->setFormatDateTimeToDB($data['end_date']);
        
        return parent::update($id, $data, $this->table);
    }

    public function getById($id, $table = null,$selectAux = null) {
        $select = "SELECT a.*,"
                . "DATE_FORMAT(a.start_date,'%m/%d/%Y %h:%i %p')as formated_start_date,"
                . "DATE_FORMAT(a.end_date,'%m/%d/%Y %h:%i %p')as formated_end_date,"              
                . "fxGetStatusName(a.status,'status','Promotion')as status_name "
                . "FROM $this->table a "
                . "WHERE a.id = '$id' "
                . "LIMIT 1";
        $result = $this->query($select);

        if ($result->num_rows > 0) {
           return $this->resultToArray($result)[0];
        }
        return null;
    }

    public function isUsedInRecord($id, array $buscarEn = null,$andWhere = null) {
        return null;
        return parent::isUsedInRecord($id, array('productos' => 'marca'));
    }

    public function getListaSelectPromotions() {
        $select = "SELECT * FROM $this->table ";
        $result = $this->query($select);

        if ($result) {
            $array = array();
            while ($row = $result->fetch_assoc()) {
                $array[$row['id']] = $row['name'];
            }
            return $array;
        }
        return null;
    }
    
     public function getListaPromotions() {
        $select = "SELECT *,"
                . "DATE_FORMAT(start_date,'%m/%d/%Y %h:%i %p')as formated_start_date,"
                . "DATE_FORMAT(end_date,'%m/%d/%Y %h:%i %p')as formated_end_date,"              
                . "fxGetStatusName(status,'status','Promotion')as status_name "
                . "FROM $this->table "
                . "ORDER BY id DESC";
        $result = $this->query($select);

        if ($result) {
           return $this->resultToArray($result);
        }
        return null;
    }      
    
    public function getListStatus(){
        $query = "SELECT * FROM codigo_status WHERE tipo = 'Promotion'";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $array = array();
            foreach($result as $status){
                $array[$status['codigo']] = $status['descripcion'];
            }
            return $array;
        }
        return null;
    }
    
    public function getNextPromotionNumber(){
        $query = "SELECT promotion_num FROM $this->table ORDER BY promotion_num DESC LIMIT 1 ";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $result = $result->fetch_object();
            $appointmentNumber = $result->promotion_num;
            $appointmentNumber++;
            return $appointmentNumber;
        }
        return 1;
    }
    
    public function getByCode($code){
        $query = "SELECT * FROM $this->table WHERE code = '$code' LIMIT 1";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            return $this->resultToArray($result)[0];
        }
        
        return null;
    }
    
    public function applyPromotion($options){
        $valid = true;
        $code = $options['promotion_code'];
        $promotionData = $this->getByCode($code);
        
        if(is_null($promotionData)){
            $this->flashmessenger->addMessage(array('danger'=>'Codigo de promocion no existe.'));
            return null;
        }
        
        $ventasDetailTempRepo = new VentaDetallesTempRepository();
        if($ventasDetailTempRepo->existAppliedPromocion($promotionData['id'])){
            $this->flashmessenger->addMessage(array('danger'=>'Esta promocion ya esta aplicada. No puede volver a aplicarla'));
            return null;
        }
        
        if($ventasDetailTempRepo->existAppliedDiscount()){
            $this->flashmessenger->addMessage(array('danger'=>'Esta promocion no puede ser aplicada, porque la venta ya tiene un descuento.'));
            return null;
        }
        
        if($promotionData){
            $startDatePromotion = new DateTime($promotionData['start_date']);
            $endtEndPromotion = new DateTime($promotionData['end_date']);
            $today = new DateTime();
            
            if($today < $startDatePromotion){
                $this->flashmessenger->addMessage(array('danger'=>'Promocion comienza el '.$startDatePromotion->format('l, F d,Y \a\t h:i A')));
                return null;
            }
                  
            if($today > $endtEndPromotion){
                $this->flashmessenger->addMessage(array('danger'=>'Promocion termino el '.$endtEndPromotion->format('l, F d,Y \a\t h:i A')));
                return null;
            }    
            
            /*Aplicar promocion*/
            $montoPromocion = (float)$options['subtotal'] * ((float)$promotionData['percentage']/100);
            $promotionData['promotion_amount'] = $montoPromocion;
            if($this->insertProductPromotion($promotionData)){
               return true;
            }            
           
        }
        
       return null;
    }
    
    public function insertProductPromotion($promotioData){
        $array = array(
            'promotion_id'=>$promotioData['id'],
            'idProducto'=>'9999999',
            'descripcion'=>'*** '.$promotioData['name'].' *** ',
            'cantidad'=>1,
            'precio'=>($promotioData['promotion_amount']) * -1,
            'precio_sin_impuestos'=>($promotioData['promotion_amount']) * -1,            
            'descuento_item_tipo'=>'porcentaje', 
            'descuento_item_valor'=>'0', 
            'descuento_item_monto'=>'0', 
            'descuento_orden_monto'=>'0', 
            'impuestos'=>'1', 
            'impuestos_tasa'=>'0', 
            'impuestos_monto'=>'0', 
            'impuestos_incluidos'=>'0',
            'status'=>'1'
        );        
        
        $repo = new VentaDetallesTempRepository;
        $repoEntity = new EntityRepository();
        
        try{
            $repoEntity->save($array, $repo->getTableName());
            return true;
        } catch (Exception $ex) {
            $this->flashmessenger->addMessage(array('danger'=>'Error al tratar de aplicar codigo de promocion.Intente nuevamente.'));
            return null;
        }        
    }
    
}