<?php
class PurchaseRepository extends EntityRepository {

    private $table = 'purchases';
    public $flashmessenger = null;
    private $options_files = array(
        'allowedExtensions'=>array('pdf'),
        'maxFileSizeAllowed'=>1000000,
        'pathToSave'=>PATH_PURCHASE_INVOICES
    );
    
    private $options = array (
        'purchase_number'=>null,
        'date' => null,
        'id_vendor' => null,
        'requested_by'=>null,
        'approved_by'=>null,
        'id_vendor'=>null,
        'reference'=>null,
        'lot'=>null,
        'credit_note'=>null,
        'credit_note_amount'=>null,
        'payment_terms_id'=>null,
        'due_date'=>null,
        'subtotal'=>null,        
        'discount_general_type'=>null,
        'discount_general'=>null,
        'total'=>null,
        'payments'=>null,
        'comments'=>null,
        'status'=>null,
        'status_payment'=>null,
        'invoice_file'=>null);
    
    private $options_aux = array(
        'vendorName'=>null,
        'formatedDate'=>null,
        'statusName'=>null,
        'token_form'=>null #Se popula con setOption desde Controller, con post de formulario
    );
    
    /*Input double y que no son hide*/
    public $inputs_double = array(
        'discount_general'
    );
    
    public function __construct() {
        if(!$this->flashmessenger instanceof FlashMessenger){
            $this->flashmessenger = new FlashMessenger();
        }
    }
    
    public function _getTranslation($text){
        return $this->flashmessenger->_getTranslation($text);
    }
    
    public function setOptions($data){
        foreach ($this->options as $option => $value){
            if(isset($data[$option])){
              $this->options[$option] = $data[$option];
            }
        }
        
        foreach ($this->options_aux as $option => $value){
            if(isset($data[$option])){
              $this->options_aux[$option] = $data[$option];
            }
        }
    }
    
    public function getBalance(){        
        return $this->options['total'] - $this->options['payments'];
    }
    
    public function getVendorName(){        
        return $this->options_aux['vendorName'];
    }
    
    public function getUserName(){        
        return $this->options_aux['userName'];
    }
    
    public function getStatusName(){        
        return $this->options_aux['statusName'];
    }
    
    public function getDate(){
        return $this->options['date'];
    }
    
    public function getFormatedDate(){
        return $this->options_aux['formatedDate'];
    }
    
    public function getFormatedDueDate(){
        return $this->options_aux['formatedDueDate'];
    }
    
    public function getVendor(){
        return $this->options['id_vendor'];
    }
    
    public function getVendorInvoice(){
        return $this->options['vendor_invoice'];
    }
    
    public function getTotal(){
        return $this->options['total'];
    }
    
    public function getComments(){
        return $this->options['comments'];
    }
    
    public function getStatus(){
        return $this->options['status'];
    }

    public function getOptions(){
        return $this->options;
    }
    
    public function getPurchaseNumber(){        
        return $this->options['purchase_number'];
    }
    
    public function getId() {
       return $this->options['id'];
    }
    
    public function getVendorInfoComplete(){
        $proveedor = new VendorRepository();        
        return $proveedor->getById($this->options['id_vendor']);
    }
    
    public function getSaldoPendiente(){
        return $this->options['total'] - $this->options['payments'];
    }
    
    public function getTokenForm(){
        return $this->options_aux['token_form'];
    }
    
     public function saveFiles($files,$idCompra,$numFactura){       
        $file = new UploadFile();
        $file->setAllowedExtensions($this->options_files['allowedExtensions']);
        $file->setMaxFileSizeAllowed($this->options_files['maxFileSizeAllowed']);
        $file->setTempFolder($this->options_files['pathToSave']);
   
        
        $rsUpload = $file->uploadFile($files['invoice_file'],null,"Factura_de_compra_".$idCompra."_".$numFactura); 
        //exit;
        $rsUpload = $file->uploadFile($files['credit_note_file'],null,"Nota_de_credito".$idCompra."_".$numFactura); 
        
        if(!$rsUpload){
            $this->flashmessenger->addMessage(array('info'=>$file->getMessageError()));  
            return null;
        }        
        
        return true;        
    }

    public function save(array $data, $table = null) {           
        $purchaseDetailsTemp = new PurchaseDetailsTempRepository();
        if(!$purchaseDetailsTemp->isThereItemsOnPurchase($this->getTokenForm())){
            $this->flashmessenger->addMessage(array('danger'=>'Debe registrar almenos un producto para guardar la compra.'));
            return null;
        }        
        
        $tools = new Tools();
        $data['purchase_number'] = $this->getNextPurchaseNum();
        $data['date'] = $tools->setFormatDateToDB($data['date']);
        $data['payments'] = 0;        
        $data['due_date'] = $tools->setFormatDateToDB($data['due_date']);
        $data['total'] = round($data['total'],2);
        $data['status'] = '1';
        $data['payments'] = 0;       
        $data['status_payment'] = 1;
       if(trim($data['credit_note_amount']) === ''){$data['credit_note_amount'] = 0;}

        $invoice_file = $data['invoice_file'];
        unset($data['invoice_file']);   
        
        $this->startTransaction();        
        $rs = parent::save($data, $this->table);        
        $idCompra = $this->getInsertId();
        $this->setLastInsertId($idCompra);//Para utilizarlo en el Controller action insert
        
        if($rs){
            if($purchaseDetailsTemp->saveDetalles($idCompra,$this->getTokenForm(),$data['purchase_number'])){ 
                
                $this->commit();                
                $purchaseDetailsTemp->truncate($this->getTokenForm());  
                
                if($invoice_file['invoice_file']['name']!='' || $invoice_file['credit_note_file']['name']!=''){          
                    $this->saveFiles($invoice_file,$idCompra,$data['reference']);
                }             
                
                return true;
            }
        }        
        
        $this->rollback();    
        $this->flashmessenger->addMessage(array(
            'error'=>$this->_getTranslation('Error. Intenta nuevamente o contacta a tu proveedor de sistemas.')));
        return null;        
    }

    
    public function delete($id, $table = null) {
        $currentData = $this->getById($id);
        if($currentData['status'] == '2'){return true;}

        $this->startTransaction();
        $rs = parent::update($id, array('status'=>'2'), $this->table);
        
        if($rs){
            $detalles = $this->getPurchaseDetailsSaved($id);
            if(!$this->subInventoryFromPurchaseDetalles($detalles,$currentData['purchase_number'])){
                $this->rollback();
                return null;
            }
            $this->commit();
            return true;
        }

        $this->rollback();
        return null;
    }
    
    public function subInventoryFromPurchaseDetalles($detallesAfectados,$purchase_number){        
        $repoInventario = new SupplyInventoryRepository();         
        $array = array();
        
        foreach($detallesAfectados as $detalle){     
            if($detalle['llevar_inventario'] == 'Si'){
                $row = array(                        
                        'supply_id'=>$detalle['id_product'],
                        'quantity'=>$detalle['quantity'],
                        'location_id'=>$detalle['location_id'],
                        'controller'=>'Compra-eliminar '.$purchase_number
                    );
                $array[] = $row;
            }            
        }

        return $repoInventario->deleteAddInventory($array);        
    }

    public function update($id, $data, $table = null) {      
        $purchaseDetailsTemp = new PurchaseDetailsTempRepository();
        if(!$purchaseDetailsTemp->isThereItemsOnPurchase($this->getTokenForm())){
            $this->flashmessenger->addMessage(array('danger'=>'Debe registrar almenos un producto para guardar la compra.'));
            return null;
        }        
       
        $tools = new Tools();
        $data['date'] = $tools->setFormatDateToDB($data['date']); 
        $data['due_date'] = $tools->setFormatDateToDB($data['due_date']);
        $data['total'] = round($data['total'],2);
       if(trim($data['credit_note_amount']) === ''){$data['credit_note_amount'] = 0;}
        
        if(trim($data['status']) == ''){unset($data['status']);}
        $invoice_file = $data['invoice_file'];        
       
        unset($data['payments'],$data['status_payment'],$data['invoice_file'],$data['purchase_number']);       
       
        $currentData = $this->getById($id);
        $this->startTransaction();
        $result = parent::update($id, $data, $this->table);        
        if($result){
            $purchaseDetailsTemp = new PurchaseDetailsTempRepository();
           
            if($purchaseDetailsTemp->updateDetalles($id,$this->getTokenForm(),$currentData['purchase_number'])){                   

                $this->commit();               
                $purchaseDetailsTemp->truncate($this->getTokenForm());   
                
                if($invoice_file['invoice_file']['name']!='' || $invoice_file['credit_note_file']['name']!=''){          
                    $this->saveFiles($invoice_file,$id,$data['reference']);
                }

               return true;
            }
        }
        
        $this->rollback();
        return null;
    }
    
    public function updateString($fields, $where, $table = null) {
        return parent::updateString($fields, $where, $this->table);
    }
    
    public function existeReferencia($id,$reference,$vendor){
       $query = "SELECT * FROM purchases_reference_vendor "
               . "WHERE vendor_invoice = '$reference' "
               . "AND id_vendor = '$vendor' "
               . "AND id != '$id' "
               . "AND status != 2"; 
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $result = $this->resultToArray($result);
            return $result[0];
        }
        return null;
    }

    public function getById($id, $table = null,$selectAux = null) {
        $select = "SELECT *,"                
                . "fxGetPaymentTermsName(payment_terms_id)as paymentTermsName,"
                . "DATE_FORMAT(date,'%m/%d/%Y')as formatedDate,"
                . "DATE_FORMAT(due_date,'%m/%d/%Y')as formatedDueDate,"
                . "(total - IFNULL(payments,0)) as saldo_pendiente,"
                . "fxGetVendorName(id_vendor) as vendorName,"
                . "fxGetStatusName(status,'status','Purchase')as statusName, "
                . "fxGetUserName(creado_por) as userName, "
                . "DATE_FORMAT(creado_fecha,'%m/%d/%Y')as formatedCreadoFecha "
                . "FROM $this->table "
                . "WHERE id = '$id'";
        
        //echo $select;
        $result = $this->query($select);

        if ($result->num_rows>0) {
            $set = $this->resultToArray($result);
            return $set[0];
        }

        return false;
    }

    public function isUsedInRecord($id, array $buscarEn = null,$andWhere = null) {
        return null;        
    }
    
    public function crearTablaDetallesForUser(){
        $login = new Login();        
        $query = "CREATE TABLE IF NOT EXISTS purchase_details_".$login->getId()." 
                 (  
                    `token_form` char(50) NOT NULL,
                    `id` int(11) NOT NULL AUTO_INCREMENT,
                    `id_detail` int(11) NULL,
                    `id_purchase` int(11) NULL,
                    `location_id` int(11) NULL,
                    `id_product` int(11) NOT NULL,
                    `description` varchar(255) NOT NULL,
                    `description_details` TEXT NULL,
                    `quantity` double NOT NULL,
                    `cost` double NOT NULL,
                    `discount` double NOT NULL,
                    `discount_type` char(15) NOT NULL,
                    `discount_amount` double NULL DEFAULT '0',
                    `discount_general` double  NULL DEFAULT '0',
                    `discount_general_type` char(15) NOT NULL,
                    `discount_general_amount` double NULL DEFAULT '0',
                    `taxes` int(11) NOT NULL,
                    `taxes_rate` double NOT NULL,
                    `taxes_amount` double NOT NULL,
                    `amount` double NOT NULL,
                    `total` double NOT NULL,
                    PRIMARY KEY (`id`)
                 )ENGINE=InnoDB DEFAULT CHARSET=utf8;";
        
       $result = $this->query($query);
    }
    
    public function insertDetalle($data){
        $purchaseDetailsTemp = new PurchaseDetailsTempRepository();        
        return $purchaseDetailsTemp->save($data);
    }
    
    public function getPurchaseDetails($token_form){
        $purchaseDetailsTemp = new PurchaseDetailsTempRepository();        
        return $purchaseDetailsTemp->getPurchaseDetails($token_form);       
    }
    
    public function getPurchaseDetailSaved($id){
        $query = "SELECT c.*,
                    p.code as code
                    FROM purchase_details c 
                    LEFT JOIN supplie_and_service p ON c.id_product = p.id
                    WHERE c.id = '$id'";
        $result = $this->query($query);
        
        if($result->num_rows >0){
            $result = $this->resultToArray($result)[0];
            return $result;
        }
        
        return null;
    }
    
    public function getPurchaseDetailsSaved($id){
        $query = "SELECT c.*,
                    p.code as code,
                    fxGetImpuestoName(c.taxes)as taxName,
                    p.llevar_inventario
                    FROM purchase_details c 
                    LEFT JOIN supplie_and_service p ON c.id_product = p.id
                    WHERE c.id_purchase = '$id'";
        $result = $this->query($query);
        
        if($result){
            $result = $this->resultToArray($result);
            return $result;
        }
        
        return null;
    }
    
    public function setPurchaseDetailsById($idCompra,$tokenForm){
        $repository = new PurchaseDetailsTempRepository();
        
        return $repository->setPurchaseDetailsById($idCompra,$tokenForm);
    }
    
    public function getProductById($idProducto){
        $query = "SELECT * FROM supplie_and_service WHERE id = '$idProducto' LIMIT 1";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $result = $this->resultToArray($result);
            return $result[0];
        }        
        return null;
    }
    
    public function getListPurchase(){         
        $query = "SELECT c.*,
                DATE_FORMAT(c.date,'%m/%d/%Y')as date,
                (total - payments)as balance,
                fxGetStatusName(c.`status`,'status','Purchase')as statusName,
                fxGetVendorName(c.id_vendor) as vendor
                FROM $this->table c
                WHERE  1=1 "
              . "GROUP BY c.id "
              . "ORDER BY c.id DESC ";
        
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        
        return null;
  }
    
    public function getDueDate($date,$creditDays){
        $query = "SELECT DATE_ADD('{$date}',INTERVAL $creditDays DAY)as due_date";
        $result = $this->query($query);
        
        if($result){
            $result = $result->fetch_object();
            return $result->due_date;
        }
        return '';
    }
    
    public function getListFacturaPendientesByProveedor($options){                    
        $idProveedor =  $options['proveedor'];
        $query = "SELECT f.*,
                  fxGetVendorName(f.id_vendor)as proveedorName,
                  DATE_FORMAT(convert(substring(f.date,1,10),date),'%m/%d/%Y')as fecha,
                  DATE_FORMAT(due_date,'%m/%d/%Y') as fecha_pago,
                  (total - IFNULL(payments,0))as saldo_pendiente
                  FROM purchases f
                  WHERE  1=1
                  AND status_payment = '1'
                  AND id_vendor = '$idProveedor' "
                . "GROUP BY f.id "
                . "ORDER BY f.id ASC";

        $result = $this->query($query);
        
        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        
        return null;
  } 
  
    public function getLotsLike($item){
        $query = "SELECT lot FROM manifest WHERE lot like '%$item%' ORDER BY lot";
        $result = $this->query($query);
        
        if($result){
            return $this->resultToArray($result);
        }
        
        return null;
    }
    
     public function getPurchaseNumberById($id){
        $query = "SELECT purchase_number FROM $this->table WHERE id = '$id'";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $result = $result->fetch_object();
            return $result->purchase_number;
        }
    }
    
    public function getListFiles($id){       
        $fileManagement = new FileManagement();
        return $fileManagement->getStringListFilesByOperationAndPrefix('purchase',$id);        
    }
    
    public function getListStatus(){
        $query = "SELECT * FROM status_code WHERE operation = 'PurchaseGoodAndService'";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $array = array();
            foreach($result as $status){
                $array[$status['code']] = $status['description'];
            }
            return $array;
        }
        return null;
    }
    
     public function getPurchaseGoodAndServiceDetailSaved($id){
        $query = "SELECT c.*,
                    pu.purchase_number
                    FROM purchase_details c 
                    LEFT JOIN purchases pu ON c.id_purchase = pu.id
                    LEFT JOIN supplie_and_service p ON c.id_product = p.id
                    WHERE c.id = '$id'";
        $result = $this->query($query);
        
        if($result){
            $result = $this->resultToArray($result)[0];
            return $result;
        }
        
        return null;
    }
    
    public function getNextPurchaseNum(){
        $query = "SELECT purchase_number FROM purchases ORDER BY purchase_number DESC LIMIT 1";        
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $result = $result->fetch_object();
            $purchaseNum = $result->purchase_number;
            $purchaseNum = $purchaseNum + 1;
            return $purchaseNum;
        }else{
            return 1;
        }
    }
    
    public function createFilterFecha($options,$campoFecha = null ){
        if(!isset($options['startDate']) && !isset($options['endDate'])){return null;}        
        $startDate = $options['startDate'];
        $endDate = $options['endDate'];
        $fecha = null;
        $tools = new Tools();
        if($startDate!=null){
            $startDate = $tools->setFormatDateToDB($startDate);
            if($endDate!=null){
                $endDate = $tools->setFormatDateToDB($endDate);
                $fecha .=" AND $campoFecha BETWEEN '{$startDate}' AND '{$endDate}' ";
            }else{
                $fecha .=" AND $campoFecha BETWEEN '{$startDate}' AND '{$startDate}' ";
            }
        }elseif($endDate!=null){
            $fecha .=" AND $campoFecha BETWEEN '{$endDate}' AND '{$endDate}' ";
        }
        
        return $fecha;
    }
}