<?php
class StoreStockTemplateRepository extends EntityRepository {

    private $table = 'stores_stock_templates';
    public $flashmessenger = null;
    
    private $options = array (
        'template_name'=>null,
        'status'=>null,
        'comments'=>null);
    
    private $options_aux = array(
        'statusName'=>null,
        'userName'=>null,
        'token_form'=>null
    );
    
    public function setOptions($data){
        foreach ($this->options as $option => $value){
            if(isset($data[$option])){
              $this->options[$option] = $data[$option];
            }
        }
        
        foreach ($this->options_aux as $option => $value){
            if(isset($data[$option])){
              $this->options_aux[$option] = $data[$option];
            }
        }
    }

    public function getOptions(){
        return $this->options;
    }
    
    public function getTokenForm(){
        return $this->options_aux['token_form'];
    }
    
   public function getNombre(){
        return $this->options['nombre'];
    }
    
    public function __construct() {
        if(!$this->flashmessenger instanceof FlashMessenger){
            $this->flashmessenger = new FlashMessenger();
        }
    }
  
    public function _getTranslation($text){
        return $this->flashmessenger->_getTranslation($text);
    }

    public function save(array $data, $table = null) {           
        $data['status'] = '1';
        
        $this->startTransaction();        
        parent::save($data, $this->table);        
        
        $storeInDetailsTemp = new StoreStockTemplateDetailsTempRepository();
        $idCompra = $this->getInsertId();
        $this->setLastInsertId($idCompra);//Para utilizarlo en el Controller action insert
        
        if($storeInDetailsTemp->saveDetalles($idCompra, $this->getTokenForm())){   
            $this->commit();
            $storeInDetailsTemp->truncate($this->getTokenForm());  

            return true;
        }
        
        $this->rollback();   
        $this->flashmessenger->addMessage(array(
            'error'=>$this->_getTranslation('Error. Intenta nuevamente o contacta a tu proveedor de sistemas.')));
        return null;        
    }
    
    public function delete($id, $table = null) {
        //return parent::delete($id, $this->table);
        return parent::update($id, array('status'=>'2'), $this->table);
    }

    public function update($id, $data, $table = null) {              
        $this->startTransaction();      
        $result = parent::update($id, $data, $this->table);
        
        if($result){
            $repository = new StoreStockTemplateDetailsTempRepository();
            if($repository->updateDetalles($id, $this->getTokenForm())){                                
                $this->commit();
                $repository->truncate($this->getTokenForm());    
            
                return true;
            }
        }
        
        $this->rollback();
        return null;
    }
    
    public function updateString($fields, $where, $table = null) {
        return parent::updateString($fields, $where, $this->table);
    }

    public function getById($id, $table = null,$selectAux = null) {
        $select = "SELECT *,"
                . "fxGetUserName(creado_por)as userName,"
                . "fxGetStatusName(status,'status','StoreStockTemplate')as statusName "
                . "FROM $this->table "
                . "WHERE id = '$id'";
        $result = $this->query($select);

        if ($result->num_rows>0) {
            $set = $this->resultToArray($result);
            return $set[0];
        }

        return false;
    }

    public function isUsedInRecord($id, array $buscarEn = null,$andWhere = null) {
        return null;
        return parent::isUsedInRecord($id, array('store-out' => 'requisition'));
    }
    
    public function crearTablaDetallesForUser(){
        $login = new Login();
        
        $query = "CREATE TABLE IF NOT EXISTS stores_stock_template_details_".$login->getId()." 
                 (  
                    `token_form` char(50) NOT NULL,
                    `id` int(11) NOT NULL AUTO_INCREMENT,
                    `id_detail` int(11) NULL,
                    `store_stock_template_id` int(11) NULL,
                    `id_product` int(11) NOT NULL,
                    `days` text NOT NULL,
                    `min` double NOT NULL,
                    `max` double NOT NULL,
                    PRIMARY KEY (`id`)
                 )ENGINE=InnoDB DEFAULT CHARSET=utf8;";
        
       $result = $this->query($query);
    }
    
    public function insertDetalle($data){
        $storeInDetailsTemp = new StoreStockTemplateDetailsTempRepository();
        
        return $storeInDetailsTemp->save($data);
    }
    
    public function getStoreDetails($token_form){
        $login = new Login();
        $query = "SELECT v.*,
                    v.id as idDetailTemp,
                    p.id as product,
                    p.codigo as code,
                    p.descripcion as description,
                    fxGetTamanoDescripcion(p.tamano)as size
                  FROM stores_stock_template_details_".$login->getId()." v LEFT JOIN productos p ON v.id_product = p.id
                  WHERE token_form = '$token_form' ORDER BY v.id";
        $result = $this->query($query);
        
        if($result){
            $result = $this->resultToArray($result);
            return $result;
        }
        
        return null;
    }
    
    public function getStoreDetailsSaved($id){
        $query = "SELECT c.*,
                    p.codigo as code,
                    p.descripcion as description,
                    fxGetTamanoDescripcion(p.tamano)as size
                    FROM stores_stock_template_details c 
                    LEFT JOIN productos p ON c.id_product = p.id
                    WHERE c.store_stock_template_id = '$id'";

        $result = $this->query($query);
        
        if($result){
            $result = $this->resultToArray($result);
            return $result;
        }
        
        return null;
    }
    
    public function setStoreStockTemplateDetailsById($idCompra, $tokenForm){
        $repository = new StoreStockTemplateDetailsTempRepository();
        
        return $repository->setStoreDetailsById($idCompra, $tokenForm);
    }
    
    public function truncateIfIsEditInfo(){
        $repository = new StoreStockTemplateDetailsTempRepository();
        $repository->truncateIfIsEditInfo();
        
    }
    
    public function getProductById($idProducto){
        $query = "SELECT * FROM productos WHERE id = '$idProducto' LIMIT 1";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $result = $this->resultToArray($result);
            return $result[0];
        }        
        return null;
    }
    
    public function getListStoresStockTemplate(){          
        $query = "SELECT r.*,
                SUM(d.min)as quantity,
                fxGetStatusName(r.`status`,'status','StoreStockTemplate')as status_name,
                fxGetUserName(r.creado_por) as user
                FROM stores_stock_templates r 
                LEFT JOIN stores_stock_template_details d ON r.id = d.store_stock_template_id " 
              . "GROUP BY r.id "
              . "ORDER BY r.id DESC ";
    
        $result = $this->query($query);

        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        
        return null;
  }
  
  public function getListSelectStores(){          
        $query = "SELECT r.*,
                SUM(d.min)as quantity,
                fxGetStatusName(r.`status`,'status','StoreStockTemplate')as statusName,
                fxGetUserName(r.creado_por) as user
                FROM stores r LEFT JOIN stores_stock_template_details d ON r.id = d.id_store " 
              . "GROUP BY r.id "
              . "ORDER BY r.nombre ASC ";
    
        $result = $this->query($query);
        
       if ($result) {
            $array = array();
            while ($row = $result->fetch_assoc()) {
                $array[$row['id']] = $row['nombre'];
            }
            return $array;
        }
        return null;
  }
  
  public function getListDays(){
      return array(
          '1'=>'Lun',
          '2'=>'Mar',
          '3'=>'Mie',
          '4'=>'Jue',
          '5'=>'Vie',
          '6'=>'Sab',
          '7'=>'Dom',
      );
  }
  
  public function daysArrayTostring($days){
      $stringDays = '';
      foreach($days as $day){
          $stringDays .= $day.",";
      }
      
      return trim($stringDays,',');
  }
  
  public function getListTemplateStoreStock(){
        $query = "SELECT id,template_name FROM $this->table WHERE status = 1 ORDER BY template_name";
        $result = $this->query($query);
        
        if($result){
            $array = array();
            while($row = $result->fetch_object()){
                $array[$row->id] = $row->template_name;
            }
            
            return $array;
        }
        
        return null;
    }
    
    public function createFromStoreStock($store_id,$template_name){        
        $data = array(
            'template_name'=>$template_name,
            'status'=>1
            );
        
        $rs = parent::save($data, $this->table);        
        $idRequisitionTemplate = $this->getInsertId();        
        
        if($rs){
            $requisitionRepo = new SucursalRepository();
            $requisitionDetails = $requisitionRepo->getStoreDetailsSaved($store_id);
            
            if($requisitionDetails){
                foreach($requisitionDetails as $detail){
                    $detail['store_stock_template_id'] = $idRequisitionTemplate;
                    unset($detail['id'],$detail['id_store'],$detail['code'],$detail['description'],$detail['size'],$detail['status'],$detail['creado_fecha'],$detail['creado_por'],$detail['ultima_mod_por'],$detail['ultima_mod_fecha'],$detail['modificaciones']);
                    parent::save($detail,'stores_stock_template_details');
                }
            }
        }       
        
        return true;
    }
    
    public function createFilterFecha($options,$campoFecha = null ){
        if(!isset($options['startDate']) && !isset($options['endDate'])){return null;}        
        $startDate = $options['startDate'];
        $endDate = $options['endDate'];
        $fecha = null;
        $tools = new Tools();
        if($startDate!=null){
            $startDate = $tools->setFormatDateToDB($startDate);
            if($endDate!=null){
                $endDate = $tools->setFormatDateToDB($endDate);
                $fecha .=" AND $campoFecha BETWEEN '{$startDate}' AND '{$endDate}' ";
            }else{
                $fecha .=" AND $campoFecha BETWEEN '{$startDate}' AND '{$startDate}' ";
            }
        }elseif($endDate!=null){
            $fecha .=" AND $campoFecha BETWEEN '{$endDate}' AND '{$endDate}' ";
        }
        
        return $fecha;
    }
}