<?php
class SupplieOutRepository extends EntityRepository {

    private $table = 'supplie_outs';
    public $flashmessenger = null;
    private $options = array (
        'id'=>null,
        'date' => null,
        'requested_by'=>null,
        'comments'=>null,
        'status'=>null
    );
    
    private $options_aux = array(
        'statusName'=>null,
        'formatedDate'=>null,
        'userName'=>null
    );
    
    public function __construct() {
        if(!$this->flashmessenger instanceof FlashMessenger){
            $this->flashmessenger = new FlashMessenger();
        }
    }
    
    public function setOptions($data){
        foreach ($this->options as $option => $value){
            if(isset($data[$option])){
              $this->options[$option] = $data[$option];
            }
        }
        
        foreach ($this->options_aux as $option => $value){
            if(isset($data[$option])){
              $this->options_aux[$option] = $data[$option];
            }
        }
    }    
    
    public function getOptions(){
        return $this->options;
    }
    
    public function _getTranslation($text){
        return $this->flashmessenger->_getTranslation($text);
    }
    
    public function getDate(){
        return $this->options['date'];
    }
    
    public function getFormatedDate(){
        $fecha = substr($this->getDate(), 0, 10);
        $fecha = strftime('%d/%m/%Y',  strtotime($fecha));
        return $fecha;
    }
    
    public function getRequestedBy(){
        return $this->options['requested_by'];
    }  
    
    public function getStatus(){
        return $this->options['status'];
    }
    
    public function getStatusName(){
        return $this->options_aux['statusName'];
    }
    
    public function getUserName(){
        return $this->options_aux['userName'];
    }
    
    public function getComments(){
        return $this->options['comments'];
    }

    public function save(array $data, $table = null) {      
        $tools = new Tools();
        $data['date'] = $tools->setFormatDateToDB($data['date']);
        $data['status'] = '1';
        unset($data['id']);
        
        $this->startTransaction();        
        parent::save($data, $this->table);        
        
        $storeInDetailsTemp = new SupplieOutDetailsTempRepository();
        $idSupplieOut = $this->getInsertId();
        $this->setLastInsertId($idSupplieOut);//Para utilizarlo en el Controller action insert
        
        if($storeInDetailsTemp->saveDetalles($idSupplieOut)){              
            $this->commit();
            $storeInDetailsTemp->drop();
        
            return true;
        }
        
        $this->rollback();    
        $this->flashmessenger->addMessage(array(
            'error'=>$this->_getTranslation('Error. Intenta nuevamente o contacta a tu proveedor de sistemas.')));
        return null;        
    }
    
    public function delete($id, $table = null) {
        $currentData = $this->getById($id);
        if($currentData['status'] == '2'){return true;}
        
        $this->startTransaction();
        $rs = parent::update($id, array('status'=>'2'), $this->table);    
        if($rs){
            $repoSupplieInventory = new InventorySupplieRepository();
            $details = $this->getSupplieOutDetailsSaved($id);
            if($details){                
                foreach($details as $detail){
                    $detail['controller'] = "Salida - Eliminar {$id}";
                    if(!$repoSupplieInventory->addInventory($detail)){
                        $this->rollback();
                        return null;
                    }
                }                
            }
            $this->commit();
            return true;
        }
        $this->rollback();
        return null;
    }

    public function update($id, $data, $table = null) {              
        $tools = new Tools();
        $data['date'] = $tools->setFormatDateToDB($data['date']);
        if(trim($data['status']) == ''){unset($data['status']);}       
        
        $this->startTransaction();
        //Actualizo tabla supplie_outs
        $result = parent::update($id, $data, $this->table);        
        if($result){
            $repository = new SupplieOutDetailsTempRepository();
            if($repository->updateDetalles($id)){                      
                $this->commit();
                $repository->drop();   
                return true;
            }
        }
        
        $this->rollback();
        return null;
    }
    
    public function updateString($fields, $where, $table = null) {
        return parent::updateString($fields, $where, $this->table);
    }

    public function getById($id, $table = null,$selectAux = null) {
        $select = "SELECT *,"
                . "fxGetStatusName(status,'status','SupplieOut')as statusName, "
                . "fxGetUserName(creado_por)as userName "
                . "FROM $this->table "
                . "WHERE id = '$id'";
        $result = $this->query($select);

        if ($result->num_rows>0) {
            $set = $this->resultToArray($result);
            return $set[0];
        }

        return false;
    }

    public function isUsedInRecord($id, array $buscarEn = null,$andWhere = null) {
        return null;
        return parent::isUsedInRecord($id, array('compras' => 'id')," AND status = 2 AND (type != 'BienesyServicios' AND type != 'Consigna' AND type != 'MateriaPrima')");
    }
    
    public function crearTablaDetallesForUser(){
        $login = new Login();
        $query = "DROP TABLE IF EXISTS supplie_out_details_".$login->getId();
        $this->query($query);
        
        $query = "CREATE TABLE IF NOT EXISTS supplie_out_details_".$login->getId()." 
                 (  `id` int(11) NOT NULL AUTO_INCREMENT,
                    `id_detail` int(11) NULL,
                    `id_supplie_out` int(11) NULL,  
                    `idProduct` int(11) NOT NULL,
                    `quantity` double NOT NULL,
                    `location` int(11) NOT NULL,                    
                    PRIMARY KEY (`id`)
                 )ENGINE=InnoDB DEFAULT CHARSET=utf8;";
        
       $result = $this->query($query);
    }
    
    public function insertDetalle($data){
        $storeOutDetailsTemp = new SupplieOutDetailsTempRepository();
        
        return $storeOutDetailsTemp->save($data);
    }
    
    public function getSupplieOutDetails(){
        $login = new Login();
        $query = "SELECT v.*,
                    v.id as idDetailTemp,
                    p.code,
                    p.description,
                    fxGetMarcaDescripcion(p.brand_id)as brand,
                    fxGetPresentacionDescripcion(p.presentation_id)as presentation,
                    fxGetLocationDescription(v.location)as location
                  FROM supplie_out_details_".$login->getId()." v 
                  LEFT JOIN supplie_and_service p ON v.idProduct = p.id
                  ORDER BY v.id";
        $result = $this->query($query);
        
        if($result){
            $result = $this->resultToArray($result);
            return $result;
        }
        
        return null;
    }
    
    public function getSupplieOutDetailSaved($id){
        $query = "SELECT c.*,
                    p.code as code
                    FROM supplie_out_details c 
                    LEFT JOIN supplie_and_service p ON c.idProduct = p.id
                    WHERE c.id = '$id' "
                ;
        $result = $this->query($query);
        
        if($result){
            $result = $this->resultToArray($result)[0];
            return $result;
        }
        
        return null;
    }
    
    public function getSupplieOutDetailsSaved($id,$idProduct = null){
        if($idProduct){ $idProduct = " AND c.idProduct = $idProduct";}
        $query = "SELECT c.*,
                    p.code as code,
                    p.description,
                    fxGetMarcaDescripcion(p.brand_id)as brandName,
                    fxGetPresentacionDescripcion(p.presentation_id)as presentationName,
                    fxGetLocationDescription(c.location)as locationName
                    FROM supplie_out_details c 
                    LEFT JOIN supplie_and_service p ON c.idProduct = p.id
                    WHERE c.id_supplie_out = '$id' $idProduct"
                ;
        $result = $this->query($query);
        
        if($result){
            $result = $this->resultToArray($result);
            return $result;
        }
        
        return null;
    }
    
    public function setSupplieOutDetailsById($idCompra){
        $repository = new SupplieOutDetailsTempRepository();
        
        return $repository->setSupplieOutDetailsById($idCompra);
    }
    
    public function truncateIfIsEditInfo(){
        $repository = new SupplieOutDetailsTempRepository();
        $repository->truncateIfIsEditInfo();
        
    }
    
    public function getSupplieById($idSupplie){
        $query = "SELECT * FROM supplie_and_service WHERE id = '$idSupplie' LIMIT 1";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $result = $this->resultToArray($result);
            return $result[0];
        }        
        return null;
    }
    
    
    public function getListSupplieOut(){          
        $query = "SELECT c.*,
                DATE_FORMAT(c.date,'%d/%m/%Y')as date,
                fxGetStatusName(c.`status`,'status','SupplieOut')as statusName,
                fxGetUserName(c.creado_por)as user
                FROM supplie_outs c
                LEFT JOIN supplie_out_details d ON c.id = d.id_supplie_out
                WHERE  1=1 " 
              . "GROUP BY c.id "
              . "ORDER BY c.id DESC ";

    
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        
        return null;
  }
  
  public function getListSelectLocations(){
        $query = "SELECT * FROM locations ORDER BY id ASC";
        $result = $this->query($query);
        
        if($result){
            $array = array();
            while($row = $result->fetch_object()){
                $array[$row->id] = $row->name;
            }
            
            return $array;
        }
        
        return null;
    }
    
    public function createFilterFecha($options,$campoFecha = null ){
        if(!isset($options['startDate']) && !isset($options['endDate'])){return null;}        
        $startDate = $options['startDate'];
        $endDate = $options['endDate'];
        $fecha = null;
        $tools = new Tools();
        if($startDate!=null){
            $startDate = $tools->setFormatDateToDB($startDate);
            if($endDate!=null){
                $endDate = $tools->setFormatDateToDB($endDate);
                $fecha .=" AND $campoFecha BETWEEN '{$startDate}' AND '{$endDate}' ";
            }else{
                $fecha .=" AND $campoFecha BETWEEN '{$startDate}' AND '{$startDate}' ";
            }
        }elseif($endDate!=null){
            $fecha .=" AND $campoFecha BETWEEN '{$endDate}' AND '{$endDate}' ";
        }
        
        return $fecha;
    }
}