<?php
class SupplieReturnRepository extends EntityRepository {

    private $table = 'supplie_returns';
    public $flashmessenger = null;
    private $options = array (
        'date'=>null,
        'returned_by'=>null,
        'comments'=>null,
        'status'=>null);#lo utilizo como auxiliar, es utilizado en ReturnRepository->save llamando ReturnRepo->setDeliveryQuantity
    
    private $options_aux = array(
        'statusName'=>null,
        'userName'=>null
    );
    
    public function __construct() {
        if(!$this->flashmessenger instanceof FlashMessenger){
            $this->flashmessenger = new FlashMessenger();
        }
    }
    
    public function setOptions($data){
        foreach ($this->options as $option => $value){
            if(isset($data[$option])){
              $this->options[$option] = $data[$option];
            }
        }
        
        foreach ($this->options_aux as $option => $value){
            if(isset($data[$option])){
              $this->options_aux[$option] = $data[$option];
            }
        }
    }
    
    public function getStatus(){
        return $this->options['status'];
    }

    public function getOptions(){
        return $this->options;
    }
    
    public function getStatusName(){
        return $this->options_aux['statusName'];
    }
    
    public function getUserName(){
        return $this->options_aux['userName'];
    }
    
    public function getFormatedDate(){
        $fecha = strftime('%d/%m/%Y',  strtotime($this->options['date']));
        return $fecha;
    }
    
    public function getReturnedBy(){
        return $this->options['returned_by'];
    }  
    
    public function getComments(){
        return $this->options['comments'];
    }
    
    public function _getTranslation($text){
        return $this->flashmessenger->_getTranslation($text);
    }

    public function save(array $data, $table = null) {
        $tools = new Tools();
        $data['date'] = $tools->setFormatDateToDB($data['date']);
        $data['status'] = '1';
        unset($data['id']);
        
        $this->startTransaction();        
        parent::save($data, $this->table);        
        
        $returnDetailsTemp = new SupplieReturnDetailsTempRepository();
        $idReturn = $this->getInsertId();
        $this->setLastInsertId($idReturn);//Para utilizarlo en el Controller action insert
        
        if($returnDetailsTemp->saveDetalles($idReturn)){              
            $this->commit();
            $returnDetailsTemp->drop();

            return true;
        }
        
        $this->rollback();    
        $this->flashmessenger->addMessage(array(
            'error'=>$this->_getTranslation('Error. Intenta nuevamente o contacta a tu proveedor de sistemas.')));
        return null;        
    }
    
    public function delete($id, $table = null) {
        $currentData = $this->getById($id);
        if($currentData['status'] == '2'){return true;}
        
        $this->startTransaction();
        $result = parent::update($id, array('status'=>'2'), $this->table);            
        if($result){
            $repoSupplieInventory = new InventorySupplieRepository();
            $detalles = $this->getReturnDetailsSaved($id);
            foreach($detalles as $detalle){
                $detalle['controller'] = "Retorno - Eliminar {$id}";
                if(!$repoSupplieInventory->subInventory($detalle)){
                    $this->rollback();
                    return null;
                }
            }
            $this->commit();
            return true;
        }
    }

    public function update($id, $data, $table = null) {
        $tools = new Tools();
        $data['date'] = $tools->setFormatDateToDB($data['date']);
        if(trim($data['status']) == ''){unset($data['status']);}
       
        $this->startTransaction();
        $result = parent::update($id, $data, $this->table);        
        if($result){
            $repository = new SupplieReturnDetailsTempRepository();
            if($repository->updateDetalles($id)){              
                $this->commit();
                $repository->drop();   
                return true;
            }
        }
        
        $this->rollback();
        return null;
    }
    
    public function updateString($fields, $where, $table = null) {
        return parent::updateString($fields, $where, $this->table);
    }

    public function getById($id, $table = null,$selectAux = null) {
        $select = "SELECT *,"
                . "fxGetStatusName(status,'status','return')as statusName, "
                . "fxGetUserName(creado_por)as userName "
                . "FROM $this->table "
                . "WHERE id = '$id'";
        $result = $this->query($select);

        if ($result->num_rows>0) {
            $set = $this->resultToArray($result);
            return $set[0];
        }

        return false;
    }

    public function isUsedInRecord($id, array $buscarEn = null,$andWhere = null) {
        return null;
        return parent::isUsedInRecord($id, array('compras' => 'id')," AND status = 2 AND (type != 'BienesyServicios' AND type != 'Consigna' AND type != 'MateriaPrima')");
    }
    
    public function crearTablaDetallesForUser(){
        $login = new Login();
        $query = "DROP TABLE IF EXISTS supplie_returns_details_".$login->getId();
        $this->query($query);
        
        $query = "CREATE TABLE IF NOT EXISTS supplie_returns_details_".$login->getId()." 
                 (  `id` int(11) NOT NULL AUTO_INCREMENT,
                    `id_detail` int(11) NULL,
                    `id_return` int(11) NULL,
                    `idProduct` int(11) NULL,
                    `quantity` double NOT NULL,
                    `location` int(11) NULL,
                    PRIMARY KEY (`id`)
                 )ENGINE=InnoDB DEFAULT CHARSET=utf8;";
        
       $result = $this->query($query);
    }
    
    public function insertDetalle($data){
        $returnDetailsTemp = new SupplieReturnDetailsTempRepository();        
        return $returnDetailsTemp->save($data);
    }
    
    public function getReturnDetails(){
        $login = new Login();
        $query = "SELECT v.*,
                    v.id as idDetailTemp,
                    p.code,
                    p.description,
                    fxGetMarcaDescripcion(p.brand_id)as brand,
                    fxGetPresentacionDescripcion(p.presentation_id)as presentation,
                    fxGetLocationDescription(v.location)as location
                  FROM supplie_returns_details_".$login->getId()." v 
                  LEFT JOIN supplie_and_service p
                  ON v.idProduct = p.id
                  ORDER BY v.id";
        $result = $this->query($query);
        
        if($result){
            $result = $this->resultToArray($result);
            return $result;
        }
        
        return null;
    }
    
    public function getReturnDetailSaved($id){
        $query = "SELECT r.*
                    FROM supplie_returns_details  r
                    WHERE 1 = 1
                    AND r.id = '$id' ";
        
        $result = $this->query($query);
        
        if($result){
            return $this->resultToArray($result)[0];            
        }
        
        return null;
    }
    
    public function getReturnDetailsSaved($id,$idProduct = null){
        if($idProduct){ $idProduct = " AND c.idProduct = $idProduct";}
        $query = "SELECT c.*,
                    p.code as code,
                    p.description,
                    fxGetMarcaDescripcion(p.brand_id)as brandName,
                    fxGetPresentacionDescripcion(p.presentation_id)as presentationName,
                    fxGetLocationDescription(c.location)as locationName
                    FROM supplie_returns_details c 
                    LEFT JOIN supplie_and_service p ON c.idProduct = p.id
                    WHERE c.id_return = '$id' $idProduct"
                ;
        $result = $this->query($query);
        
        if($result){
            $result = $this->resultToArray($result);
            return $result;
        }
        
        return null;
    }
    
    public function setReturnDetailsById($idReturn){
        $repository = new SupplieReturnDetailsTempRepository();
        
        return $repository->setReturnDetailsById($idReturn);
    }
    
    public function truncateIfIsEditInfo(){
        $repository = new SupplieReturnDetailsTempRepository();
        $repository->truncateIfIsEditInfo();
        
    }
    
    public function getListReturn(){          
        $query = "SELECT *,
                DATE_FORMAT(date,'%d/%m/%Y')as date,
                fxGetStatusName(status,'status','SupplieReturn')as statusName,
                fxGetUserName(creado_por)as userName
                FROM supplie_returns
                WHERE 1=1
                ORDER BY id DESC";
        
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        
        return null;
  }
 
    public function createFilterFecha($options,$campoFecha = null ){
        if(!isset($options['startDate']) && !isset($options['endDate'])){return null;}        
        $startDate = $options['startDate'];
        $endDate = $options['endDate'];
        $fecha = null;
        $tools = new Tools();
        if($startDate!=null){
            $startDate = $tools->setFormatDateToDB($startDate);
            if($endDate!=null){
                $endDate = $tools->setFormatDateToDB($endDate);
                $fecha .=" AND $campoFecha BETWEEN '{$startDate}' AND '{$endDate}' ";
            }else{
                $fecha .=" AND $campoFecha BETWEEN '{$startDate}' AND '{$startDate}' ";
            }
        }elseif($endDate!=null){
            $fecha .=" AND $campoFecha BETWEEN '{$endDate}' AND '{$endDate}' ";
        }
        
        return $fecha;
    }
}