<?php
class VentaRepository extends EntityRepository {

    private $table = 'ventas';
    public $flashmessenger = null;
    
    public function __construct() {
        if(!$this->flashmessenger instanceof FlashMessenger){ 
            $this->flashmessenger = new FlashMessenger();
        }
    }
    
    public function _getTranslation($text){
        return $this->flashmessenger->_getTranslation($text);
    }

    public function savePos(array $data, $table = null) {
        $ventaDetallesTemp = new VentaDetallesTempRepository();
        
        if(!$ventaDetallesTemp->isThereItemsOnSale()){
            $this->flashmessenger->addMessage(array('danger'=>'Debe registrar un producto por lo menos, para guardar la venta.'));
            return null;
        }       
        
        $data['num_venta'] = $this->getNextSaleNum();
        
        /*La linea de abajo la habia puesto, para obtener el total de venta directamente de la tabla de ventasdetalles 
          Esto para evitar que se modificara usando una herramienta de debug en el navegador */
        $totalesVenta = $ventaDetallesTemp->getTotalesVenta();
        $promotios = $ventaDetallesTemp->getTotalPromocionesAplicadas();
        $data['importe'] = $totalesVenta['importe'];
        $data['descuento_items_monto'] = $totalesVenta['descuento_items_monto'];
        $data['descuento_orden_monto'] = $totalesVenta['descuento_orden_monto'];
        $data['subtotal'] = $totalesVenta['subtotal'];
        $data['total_impuestos'] = $totalesVenta['total_impuestos'];
        $data['total_venta'] = $totalesVenta['total_venta'];
        $data['promotions_code'] = $promotios['promotions_code'];
        $data['promotions_amount'] = $promotios['promotions_amount'];
        
        $login = new Login();
        $data['idCaja']= $login->getIdCaja();
        $data['idSucursal']= $login->getSucursal();
        $data['creado_por'] = $login->getId();
        $data['creado_fecha'] = date('Y-m-d H:i:s');        
        $data['status'] = '2';        
        $tools = new Tools();
        $data['fecha'] = $tools->setFormatDateToDB($data['fecha']);      
        $data['ticket'] = $tools->uniqidReal();
        
        $customerRepo = new ClienteRepository();
        $customerData = $customerRepo->getById($data['cliente']);
        $data['customer_name'] = $customerData['nombre'];        
        
        /*Ventas en linea (Social network)*/
        if($_SESSION['usuario']['tipoEstacion'] == '2'){
            $data['is_sales_online'] = 1;
            $data['plataforma'] = 'social_network';
            $data['pickup_datetime'] = $tools->setFormatDateToDB($data['pickup_datetime']);
            $data['status_pickup_online'] = 1;
            $data['printed_sale_online_status'] = '0';            
            
            $settings = new SettingsEntity();
            $data['prefix_for_sale'] = $settings->_get('sales_online_code_prefix');
        }       
        
        if($data['id']==null || $data['id']==''){unset($data['id']);}
        
        foreach($data as $key => $value){
            if($value == '' || $value === null ){
                $data[$key] = array('key'=>$key,'value'=>'NULL');                
            }
        }       
        
        $cambio = $data['cambio'];
        unset($data['cambio']);
        //echo "<pre>";var_dump($data);echo "</pre>";exit;
        $this->startTransaction();
        parent::save($data, $this->table);      
        
        $idVenta = $this->getInsertId();
        $this->setLastInsertId($idVenta); // para mostrar id en link de impresion despues de guardar venta
        
        if($ventaDetallesTemp->saveDetalles($idVenta,$data['ticket'])){
            $repositoryCobro = new CobroTempRepository();
            
            if(!$repositoryCobro->saveCobros($idVenta,$data['total_venta'],$cambio)){
                $this->rollback();
                return null;
            }           
            
            #Es pago de specialRequisition
            //if($data['id_special_requisition']!='0'){
            if(!empty($data['id_special_requisition'])){ 
                if(!$this->setAmmountPaymentsOfSpecialRequisition($data['id_special_requisition'])){
                    $this->rollback();
                    return null;
                }
            }
            
           /*REWARDS*/
            try{               
                $rewardsRepo = new RewardsRepository();
                $rewardsRepo->setCustomerBuying();           
                $rewardsRepo->descontarPointsUsed($idVenta);
                $rewardsRepo->assignPointsToCustomer($idVenta);               
                $rewardsRepo->clearCustomerBuying();
                
            } catch (Exception $ex) {
                $this->flashmessenger->addMessage(array('danger'=>$ex->getMessage()));
            }       
            
            $ventaDetallesTemp->clearDetalles();
            $repositoryCobro->clearDetalles();
            $this->commit();       
            
            return true;
        }
        
        $this->rollback();        
        return null;
    }
    
    public function suspenderVenta($data){
        $ventaDetallesTemp = new VentaDetallesTempRepository();
        
        if(!$ventaDetallesTemp->isThereItemsOnSale()){
            $this->flashmessenger->addMessage(array('danger'=>'Debe registrar un producto por lo menos, para guardar la venta.'));
            return null;
        }        
        
        $login = new Login(); 
        $data['creado_por'] = $login->getId();        
        $data['creado_fecha'] = date('Y-m-d H:i:s');
        $data['idCaja'] = $login->getIdCaja();
        $data['idSucursal'] = $login->getSucursal();
               
        $data['fecha'] = date('Y-m-d');
        $data['metodo_pago'] = 1;       
        $data['status'] = '1';        
        
        $tools = new Tools();
        $data['fecha'] = $tools->setFormatDateToDB($data['fecha']);
        unset($data['cambio'],$data['id_special_requisition']);
        
        if($data['id']==null || $data['id']==''){unset($data['id']);}
        
        foreach($data as $key => $value){
            if($value == '' || $value === null ){
                $data[$key] = array('key'=>$key,'value'=>'NULL');                
            }
        }       
        
        $this->startTransaction();
        parent::save($data, 'ventas_suspendidas');      
        
        $idVenta = $this->getInsertId();
        $this->setLastInsertId($idVenta); // para mostrar id en link de impresion despues de guardar venta
        
        if($ventaDetallesTemp->saveDetallesVentaSuspendida($idVenta)){  
            $this->commit();                       
            return true;
        }
        
        $this->rollback();        
        return null;        
    }
    
    public function delete($id, $table = null) {
        $currentData = $this->getById($id);
        if($currentData['status']== '3'){return true;}
        
        $this->startTransaction();
        $result = parent::update($id, array('status'=>'3'), $this->table);        
        
        if($result){
            $data = $this->getById($id);
            #IF id_special_requisition !=0; Es un pago de specialRequisition el que se esta eliminando
            //if($data['id_special_requisition']!='0'){
            if(!empty($data['id_special_requisition'])){ 
                if(!$this->setAmmountPaymentsOfSpecialRequisition($data['id_special_requisition'])){
                    $this->rollback();
                    return null;
                }
            }else{
                $detallesAfectados = $this->getVentaDetallesSaved($id);
                if(!$this->addInventoryFromVentasDetalles($detallesAfectados,$currentData)){
                    $this->rollback();
                    return null;
                }
            }
        }
        $this->commit();
        return true;
    }

    public function getById($id, $table = null, $selectAux = null) {
       $select = "SELECT v.*,"
               . "fxGetClienteName(v.cliente)as clienteName,"
               . "fxGetUserName(v.creado_por)as usuarioName,"
               . "fxGetFormaPagoName(v.metodo_pago)as FormaPagoName,"
               . "fxGetStatusName(v.status,'status','venta')as statusName,"
               . "fxGetStoreName(v.pickup_store)as pickupStoreName,"
               . "DATE_FORMAT(v.pickup_datetime,'%m/%d/%Y')as pickupDateTime,"
               . "e.tipo as tipoCaja "            
               . "FROM $this->table v "
               . "LEFT JOIN estaciones e ON v.idCaja = e.id "
               . "WHERE v.id = '$id'";
        $result = $this->query($select);
        
        if ($result->num_rows > 0) {
            $set = $this->resultToArray($result);
            return $set[0];
        }

        return false;
    }
    
     public function getByNumVentaAndStoreId($numVenta,$storeId) {
       $select = "SELECT v.*,"
               . "fxGetClienteName(v.cliente)as clienteName,"
               . "fxGetUserName(v.creado_por)as usuarioName,"
               . "fxGetFormaPagoName(v.metodo_pago)as FormaPagoName,"
               . "fxGetStatusName(v.status,'status','venta')as statusName "              
               . "FROM $this->table v "
               . "WHERE v.num_venta = '$numVenta' AND v.idSucursal = '$storeId'";
        $result = $this->query($select);

        if ($result->num_rows > 0) {
            $set = $this->resultToArray($result);
            return $set[0];
        }

        return false;
    }
    
    public function getByCodeTicketOnline($label){
        $settings = new SettingsEntity();
        $sales_online_code_prefix = $settings->_get('sales_online_code_prefix');
        
        $query = "SELECT v.*,"
                . "fxGetClienteName(v.cliente)as customerName, "
                . "DATE_FORMAT(v.pickup_datetime,'%m/%d/%Y %h:%i %p')as pickup_date_formated,"
                . "fxGetStoreName(v.pickup_store)as pickup_store_name,"
                . "v.total_venta as total_sale, "
                . "fxGetStatusName(v.status_pickup_online,'status_pickup_online','Ventas')as statusPickupName,"
                . "fxGetStoreName(v.delivery_sale_online_by)as delivery_sale_online_store_name,"
                . "DATE_FORMAT(v.delivery_sale_online_datetime,'%m/%d/%Y %h:%i %p')as delivery_date_formated,"
                . "fxGetUserName(v.delivery_sale_online_by)as delivery_sale_online_by_name "
                . "FROM $this->table v "
                . "WHERE CONCAT('$sales_online_code_prefix',v.id) = '$label' "
                . "AND v.is_sales_online = 1 "
                . "LIMIT 1";
        
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            return $this->resultToArray($result)[0];
        }
        return null;
    }

    public function isUsedInRecord($id, array $buscarEn = null,$andWhere = null) {
        return parent::isUsedInRecord($id, array('pfa' => 'usuario'));
    }
    
     public function activarVentaSuspendida($idVentaSuspendida){       
        $query = "SELECT * FROM ventas_suspendidas WHERE id = '$idVentaSuspendida'";
        $result = $this->query($query);
        
        if($result){            
            $repository = new VentaDetallesTempRepository();
            $repository->activarVentaSuspendida($idVentaSuspendida);
            
            parent::delete($idVentaSuspendida, 'ventas_suspendidas');
            
            $result = $this->resultToArray($result);
            return $result[0];
        }
        
        return null;
    }
    
    public function hasSaldoPendiente($id){
        $query = "SELECT saldoPendiente FROM view_saldo_pendientes_ventas WHERE idVenta = '$id'";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $result = $result->fetch_object();
            return $result->saldoPendiente;
        }
        
        return null;
    }
    
    public function crearTablaDetallesForUser(){
        $login = new Login();        
        $query = "CREATE TABLE IF NOT EXISTS ventasdetalles_".$login->getId()." 
                 (  `id` int(11) NOT NULL AUTO_INCREMENT,
                    `idVenta` int(11) NULL,
                    `promotion_id` int(11) NULL DEFAULT '0',
                    `idProducto` int(11) NOT NULL,
                    `descripcion` varchar(255) NOT NULL,
                    `cantidad` double NOT NULL,
                    `precio` double NOT NULL,
                    `precio_sin_impuestos` double NOT NULL,
                    `descuento_item_tipo` CHAR(50)  NULL,
                    `descuento_item_valor` double NULL,
                    `descuento_item_monto` double NULL,
                    `descuento_orden_monto` double NULL,
                    `impuestos` varchar(50) NULL,
                    `impuestos_tasa` double NULL,
                    `impuestos_monto` double NULL,
                    `impuestos_incluidos` CHAR(5)  NULL,
                    `status` int(11) NOT NULL,
                    PRIMARY KEY (`id`)
                 )ENGINE=InnoDB DEFAULT CHARSET=utf8;";
        
       $result = $this->query($query);
    }
    
    public function insertDetalle($data){
        $ventasDetallesTemp = new VentaDetallesTempRepository();        
        return $ventasDetallesTemp->save($data);
    }
    
    public function setDescuentoOrden($data){
        $ventasDetallesTemp = new VentaDetallesTempRepository();        
        return $ventasDetallesTemp->setDescuentoOrden($data);
    }
    
    public function getVentaDetalles(){
        $login = new Login();
        $query = "SELECT v.*,
                    v.cantidad as cantidadOrigen,
                    v.id as idDetalleTemp,                    
                    IF(idProducto != 0,v.idProducto,v.descripcion)as producto,
                    fxGetTamanoDescripcion(p.tamano) as tamanoName,
                    fxGetImpuestosTasa(v.impuestos) as impuestosTasa,
                    p.impuestos_incluidos
                  FROM ventasdetalles_".$login->getId()." v LEFT JOIN  productos p ON v.idProducto = p.id
                  WHERE v.status = '1'
                  ORDER BY v.id";
        $result = $this->query($query);
        
        if($result){
            $result = $this->resultToArray($result);
            return $result;
        }
        
        return null;
    }
    
    public function getVentasSuspendidas(){
        $login = new Login();
        $query = "SELECT *
                  FROM ventas_suspendidas
                  WHERE status = '1'    
                  AND idSucursal = '{$login->getSucursal()}'
                  ORDER BY id";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $result = $this->resultToArray($result);
            return $result;
        }
        
        return null;
    }
    
    public function getVentaDetallesSaved($id){
        $query = "SELECT vd.*,"
                . "fxGetTamanoDescripcion(p.tamano)as tamanoName,"
                . "idSucursal "
                . "FROM ventasdetalles vd "
                . "LEFT JOIN ventas v ON v.id = vd.idVenta "
                . "LEFT JOIN  productos p ON vd.idProducto = p.id "
                . "WHERE vd.idVenta = '$id'";
        $result = $this->query($query);
        
        if($result){
            $result = $this->resultToArray($result);
            return $result;
        }
        
        return null;
    }
    
    public function setVentaDetallesById($idVenta){
        $repository = new VentaDetallesTempRepository();
        
        return $repository->setVentaDetallesById($idVenta);
    }   
    
    public function setAmmountPaymentsOfSpecialRequisition($idReq){
        #Setear monto de pagos de SpecialRequisition
        $query = "SELECT *,"
                . "IFNULL(SUM(total_venta),0)as monto "
                . "FROM ventas "
                . "WHERE id_special_requisition = '$idReq' "
                . "AND status != 3 ";
        
        $result = $this->query($query);
        
        if($result){
            $result = $result->fetch_object();
            $repo = new SpecialRequisitionRepository();            
            $repo->updateString(array('ammount_payments'=>$result->monto), " id = '$idReq'");
            
            $data = $repo->getById($idReq);
            $balance = $data['ammount'] - $result->monto;
            if($balance > 0){$status = '1';}else{$status = '2';}
            $repo->updateString(array('status_payment'=>$status), " id = '$idReq'");
            
            return true;
        }        
        return null;
    }
    
    public function getPaymentsByIdSpecialRequisition($idReq){
        $query = "SELECT *,fxGetStatusName(status,'status','venta')as statusName FROM $this->table WHERE id_special_requisition = '$idReq'";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        return null;
    }
    
    
    public function truncateIfIsEditInfo(){
        $repository = new VentaDetallesTempRepository();
        $repository->truncateIfIsEditInfo();
    }
    
    public function clearDetalles(){
        /* Elimina detalles que esten activos status = 1 */
        $repository = new VentaDetallesTempRepository();
        $repository->clearDetalles();
    }
    
    public function getProductoById($idProducto){
        $repo = new ProductoRepository();
        return $repo->getById($idProducto);
    }
    
    public function getProductoByCode($code){
        $repo = new ProductoRepository();
        return $repo->getByCode($code);
    }
  
   public function getVentasByCliente($id){
       $query = "SELECT id, fxGetStatusName(status,'status','venta') as status "
                . "FROM ventas "
                . "WHERE cliente = '$id' "
                . "AND status != 3";
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        
        return null;
   }
   
    public function getCobrosByIdVenta($idVenta){
        $query = "SELECT fxGetMontoCobros('$idVenta')as cobros";
        $result = $this->query($query);
        
        if($result){
            $result = $result->fetch_object();
            return $result->cobros;
        }
        
        return null;
    }
   
    public function getCobrosDetalleByIdVenta($idVenta){
        $query = "SELECT *,SUM(monto)as monto,"
                . "fxGetFormaPagoName(formaPago)as formaPagoName "
                . "FROM cobros "
                . "WHERE idVenta = '$idVenta' "
                . "AND status != 2 "
                . "GROUP BY formaPago,numDocumento";
        $result = $this->query($query);
        
        if($result){
           return $this->resultToArray($result);
        }
        
        return null;
    }
   
   public function cajaAbierta($caja){
       $query = "SELECT acc.id "
               . "FROM apertura_cierre_cajas acc,estaciones e "
               . "WHERE acc.idCaja = e.id "
               . "AND token = '$caja' AND acc.status = '1'";
       $result = $this->query($query);
       
       if($result->num_rows > 0){
           return true;
       }
       return null;
   }
   
    public function getNextSaleNum(){
        $login = new Login();
        $query = "SELECT num_venta FROM ventas WHERE idSucursal = '{$login->getSucursal()}}' ORDER BY num_venta DESC LIMIT 1";        
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $result = $result->fetch_object();
            $numVenta = $result->num_venta;
            $numVenta = $numVenta + 1;
            return $numVenta;
        }else{
            return 1;
        }
    }
    
     public function createFilterFecha($options,$campoFecha = null ){
        if(!isset($options['fechaInicio']) && !isset($options['fechaFin'])){return null;}        
        $fechaInicio = $options['fechaInicio'];
        $fechaFin = $options['fechaFin'];
        $fecha = null;
        $tools = new Tools();
        if($fechaInicio!=null){
            $fechaInicio = $tools->setFormatDateToDB($fechaInicio);
            if($fechaFin!=null){
                $fechaFin = $tools->setFormatDateToDB($fechaFin);
                $fecha .=" AND $campoFecha BETWEEN '{$fechaInicio}' AND '{$fechaFin}' ";
            }else{
                $fecha .=" AND $campoFecha BETWEEN '{$fechaInicio}' AND '{$fechaInicio}' ";
            }
        }elseif($fechaFin!=null){
            $fecha .=" AND $campoFecha BETWEEN '{$fechaFin}' AND '{$fechaFin}' ";
        }
        
        return $fecha;
    }
    
    public function getCierreCaja($id){
        $repo = new AperturaCierreCajaRepository();
        $data = $repo->getById($id);
        
        $query = "SELECT formaPago,SUM(c.monto)as monto "
               . "FROM cobros c,ventas v "
               . "WHERE c.idVenta = v.id "
               . "AND idCaja = '{$data['idCaja']}'"
               . "AND c.creado_fecha >= '{$data['fechaApertura']}'"
               . "AND v.status != 3 "
               . "GROUP BY formaPago";   
        
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $array = array();
            foreach($result as $row){
                $array[$row['formaPago']] = $row['monto'];
            }
            return $array;
        }
        return null;
    }
    
    public function getDataVentaParaArqueo($id){
        $repo = new AperturaCierreCajaRepository();
        $data = $repo->getById($id);
        
        $query = "SELECT fxGetFormaPagoName(formaPago)as formaPago,SUM(c.monto)as monto "
               . "FROM cobros c,ventas v "
               . "WHERE c.idVenta = v.id "
               . "AND idCaja = '{$data['idCaja']}'"
               . "AND c.creado_fecha BETWEEN '{$data['fechaApertura']}' AND '{$data['fechaCierre']}'"
               . "AND v.status != 3 "
               . "GROUP BY formaPago ORDER BY formaPago ASC";   
        
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        return null;
    }
    
    public function getDataVentaParaResumen($id){
        $repo = new AperturaCierreCajaRepository();
        $data = $repo->getById($id);
        
        $query = "SELECT fxGetFormaPagoName(formaPago)as formaPago,SUM(c.monto)as monto "
               . "FROM cobros c,ventas v "
               . "WHERE c.idVenta = v.id "
               . "AND idCaja = '{$data['idCaja']}'"
               . "AND c.creado_fecha BETWEEN '{$data['fechaApertura']}' AND NOW()"
               . "AND v.status != 3 "
               . "GROUP BY formaPago ORDER BY formaPago ASC";   
        
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        return null;
    } 
    
     public function getTotalVentasByIdApertura($id){
        $repo = new AperturaCierreCajaRepository();
        $data = $repo->getById($id);
        
        $query = "SELECT SUM(c.monto)as monto "
               . "FROM cobros c,ventas v "
               . "WHERE c.idVenta = v.id "
               . "AND idCaja = '{$data['idCaja']}'"
               . "AND c.creado_fecha BETWEEN '{$data['fechaApertura']}' AND '{$data['fechaCierre']}'"
               . "AND v.status != 3 ";   
        
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $result = $result->fetch_object();
            return $result->monto;
        }
        return null;
    }
    
    public function getVentasCanceladasByIdApertura($id){
        $repo = new AperturaCierreCajaRepository();
        $data = $repo->getById($id);
        
        $query = "SELECT * "
               . "FROM ventas "
               . "WHERE idCaja = '{$data['idCaja']}' "
               . "AND creado_fecha BETWEEN '{$data['fechaApertura']}' AND '{$data['fechaCierre']}' "
               . "AND status = 3 ";   
        
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        return null;
    }
    
     public function getTotalDescuentosByIdApertura($id){
        $repo = new AperturaCierreCajaRepository();
        $data = $repo->getById($id);
        
        $query = "SELECT "
               . "ROUND(SUM(v.descuento_items_monto + v.descuento_orden_monto),2)as descuento "
               . "FROM ventas v "
               . "WHERE idCaja = '{$data['idCaja']}' "
               . "AND v.creado_fecha BETWEEN '{$data['fechaApertura']}' AND '{$data['fechaCierre']}' "
               . "AND v.status != 3 ";   
        
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $result = $result->fetch_object();
            return $result->descuento;
        }
        return null;
    }
    
    public function getSalesTodayByIdCaja($idCaja = null){        
        if($idCaja == null){
            $login = new Login();
            $idCaja = $login->getIdCaja();
        }     
        
        $fecha = $this->createFilterFecha(array('fechaInicio'=>date('Y-m-d'),'fechaFin'=>date('Y-m-d')), 'fecha');
        $query =  "SELECT *,"
                . "fxGetStatusName(status,'status','venta')as statusName "
                . "FROM ventas "
                . "WHERE 1 = 1 "
                . "$fecha "
                . "AND idCaja = '$idCaja'"
                . "ORDER BY id DESC ";
        
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        return null;
    }
    
    public function getSalesTodayByIdStore($idStore = null){        
        if($idStore == null){
            $login = new Login();
            $idStore = $login->getSucursal();
        }     
        
        $fecha = $this->createFilterFecha(array('fechaInicio'=>date('Y-m-d'),'fechaFin'=>date('Y-m-d')), 'v.fecha');
        $query =  "SELECT v.*,"
                . "GROUP_CONCAT(fxGetFormaPagoName(c.formaPago))as paymnetMethodName,"
                . "fxGetStatusName(v.status,'status','venta')as statusName "
                . "FROM ventas v "
                . "LEFT JOIN cobros c ON c.idVenta = v.id  "
                . "WHERE 1 = 1 "
                . "$fecha "
                . "AND v.idSucursal = '$idStore' "                
                . "GROUP BY v.id "
                . "ORDER BY v.id DESC ";
        
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        return null;
    }
    
     public function getDevolucionesTodayByIdCaja($idCaja = null){
        $options = array(
            'reporte'=>'devolucionesPorFecha',
            'fechaInicio'=>date('Y-m-d'),
            'fechaFin'=>date('Y-m-d'),
            'idCaja'=>$idCaja
        );
        
        if($idCaja == null){
            $login = new Login();
            $options['idCaja'] = $login->getIdCaja();
        }
        
        $repo = new ReportListasEntity();
        return $repo->getDevolucionesPorFecha($options);        
    }


    public function crearTablaCobrosForUser(){
       $repo = new CobroTempRepository();
       $repo->crearTablaCobrosForUser();
    }
    
    /*PARA CONTROL DE INVENTARIOS*/
  public function addInventoryFromVentasDetalles($detallesAfectados,$ventaData){
        $repoInventario = new InventarioRepository();      

        $sucursalId = $ventaData['idSucursal'];
        /*Si la caja es tipo '2 - En linea', el inventario se suma a la sucursaal seleccionada como 'Entregar en'*/
        if($ventaData['tipoCaja'] == '2'){
            $sucursalId = $ventaData['pickup_store'];
        }
        
        $array = array();
        foreach($detallesAfectados as $detalle){         
            $row = array(
                        'id_product'=>$detalle['idProducto'],
                        'quantity'=>$detalle['cantidad'],
                        'sucursal'=>$sucursalId,
                        'controller'=>'Venta-eliminar '.$detalle['idVenta']
                    );
            $array[] = $row;
        }

        return $repoInventario->deleteSubInventory($array);
    }
    
    
    //  Dashboard Ventas
    public function getSalesByDateRange($start,$end,$group = null,$orderBy = null){       
        if(is_null($group)){$group = " GROUP BY idSucursal";}else{$group = " GROUP BY $group";}
        if(is_null($orderBy)){$orderBy = " ORDER BY total_sales DESC";}else{$orderBy = " ORDER BY $orderBy";}
        
        $date = $this->createFilterFecha(array('fechaInicio'=>$start,'fechaFin'=>$end),'fecha');
        $query = "SELECT "
                . "DATE_FORMAT(fecha,'%m/%d/%Y %h:%i %p')as formated_date,"   
                . "fxGetStoreName(idSucursal)as store_name, "
                . "SUM(total_venta)as total_sales "
                . "FROM $this->table "
                . "WHERE 1 = 1 AND status != 3 $date $group $orderBy ";
       
        $result = $this->query($query);
        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        return null;
    }
    
    public function getSalesByStoreIdByDateRange($start,$end,$group = null,$orderBy = null){       
        $login = new Login();
        $store_id = $login->getStoreId();
        if(is_null($group)){$group = " GROUP BY DATE(fecha)";}else{$group = " GROUP BY $group";}
        if(is_null($orderBy)){$orderBy = " ORDER BY DATE(fecha) DESC";}else{$orderBy = " ORDER BY $orderBy";}
        
        $date = $this->createFilterFecha(array('startDate'=>$start,'endDate'=>$end),'DATE(date)');
        $query = "SELECT "
                . "DATE_FORMAT(fecha,'%m/%d/%Y')as date," 
                . "DAYNAME(fecha)as dayname,"
                . "fxGetStoreName(idSucursal)as store_name, "
                . "SUM(total_venta )as total_sales "
                . "FROM $this->table "
                . "WHERE 1 = 1 AND status != 3 AND idSucursal = '$store_id' $date $group $orderBy ";
        
        $result = $this->query($query);
        if($result->num_rows > 0){
            return $this->resultToArray($result);
        }
        return null;
    }
    
    public function setSaleAsDelivery($id){
        $login = new Login();
        $sucursal = $login->getSucursal();
        $date = date('Y-m-d H:i:s');
        $query = "UPDATE $this->table "
                . "SET status_pickup_online = 2,"
                . "delivery_sale_online_by = '{$login->getId()}',"
                . "delivery_sale_online_datetime = '{$date}', "
                . "delivery_sale_online_store = '{$sucursal}' "
                . "WHERE id = $id";
        $rs = $this->query($query);
        
        if($rs){
            $ventaData = $this->getById($id);
            if($ventaData['pickup_store'] != $ventaData['delivery_sale_online_store']){
                $inventoryRepo = new InventarioRepository();
                $ventaDetalles = $this->getVentaDetallesSaved($id);
                
                if($ventaDetalles){
                    foreach($ventaDetalles as $detalle){
                        $inventoryRepo->subInventory(array(
                            'id_product'=>$detalle['idProducto'],
                            'quantity'=>$detalle['cantidad'],
                            'sucursal'=>$ventaData['delivery_sale_online_store'],
                            'controller'=>'Ajuste Venta en linea - sumar '.$id
                        ));
                        
                        $inventoryRepo->addInventory(array(
                            'id_product'=>$detalle['idProducto'],
                            'quantity'=>$detalle['cantidad'],
                            'sucursal'=>$ventaData['pickup_store'],
                             'controller'=>'Ajuste Venta en linea - descontar '.$id
                        ));
                    }
                }
            }
            
            return true;
        }
        
        return null;
    }
    
    public function getSalesOnlineSNForToday(){
        $login = new Login();
        $storeId = $login->getSucursal();
        $role = $login->getRole();
        $date = date('Y-m-d');
        
        $andStoreId = '';
        if($role != '1'){$andStoreId = " AND v.pickup_store = '{$storeId}' ";}
        
        $query = "SELECT v.*,"
                . "fxGetStoreName(v.idSucursal)as store_name,"
                . "fxGetClienteName(v.cliente)as customer_name,"
                . "fxGetStoreName(v.pickup_store)as pickup_store_name,"
                . "DATE_FORMAT(v.pickup_datetime,'%m/%d/%Y')as formated_pickup_datetime, "
                . "fxGetStatusName(v.status_pickup_online,'status_pickup_online','Ventas')as status_pickup_name, "
                . "GROUP_CONCAT(fxGetFormaPagoName(c.formaPago))as payment_method_name, "
                . "fxGetStoreName(v.delivery_sale_online_by)as delivery_sale_online_store_name,"
                . "DATE_FORMAT(v.delivery_sale_online_datetime,'%m/%d/%Y %h:%i %p')as delivery_date_formated,"
                . "fxGetUserName(v.delivery_sale_online_by)as delivery_sale_online_by_name,"
                . "IF('$date' > DATE(pickup_datetime),1,0)over_due "
                . "FROM ventas v "
                . "LEFT JOIN cobros c ON c.idVenta = v.id "
                . "WHERE v.is_sales_online = '1' "
                . "AND v.status != 3 "                
                . "AND (DATE(v.pickup_datetime) = '$date' OR (DATE(v.pickup_datetime) < '$date' AND status_pickup_online = 1 ) )  "
                . "$andStoreId "
                . "GROUP BY v.id "
                . "ORDER BY v.pickup_datetime DESC ";
        
        $result = $this->query($query);
        
        if($result->num_rows > 0){
            $result = $this->resultToArray($result);
            return $result;
        }
        
        return null;
    }                

}