<?php
/**
 * Description of NivelForm
 *
 * @author carlos
 */

class NotificationForm extends Form {

    public function __construct() {
        $this->setActionForm('Notification.php');
        $this->setName('notification');
        $this->setClass('notification form-horizontal');
        $this->setMethod('post');
        parent::__construct();
        $this->init();
    }

    public function init() {
        $this->addElement(array(
            'type' => 'hidden',
            'name' => 'notification_num'
        ));
        
        $this->addElement(array(
            'type' => 'hidden',
            'name' => 'created_by_name'
        ));
        
        $this->addElement(array(
            'type' => 'hidden',
            'name' => 'created_date'
        ));
        
        $this->addElement(array(
            'type' => 'select multiple',
            'name' => 'send_to',
            'label'=>'Para',
            'multiOptions'=>$this->getListUsers(),
            'required'=>true
        )); 
        
        $this->addElement(array(
            'type' => 'text',
            'name' => 'subject',
            'label'=>'Asunto',
            'required'=> true
        ));
        
        $this->addElement(array(
            'type' => 'textarea',
            'name' => 'message',
            'label'=>'Mensaje',
            'required'=>true
        ));     
        
        $this->addElement(array(
            'type' => 'checkbox',
            'name' => 'send_now',
            'label'=>'Enviar ahora',            
            'required'=> false
        ));
        
        $this->addElement(array(
            'type' => 'text',
            'name' => 'when_send',
            'label'=>'Cuando enviar',
            'required'=> true
        ));
        
        $this->addElement(array(
            'type' => 'checkbox',
            'name' => 'repeat',
            'label'=>'Repetir',            
            'required'=> false
        ));
        
        $this->addElement(array(
            'type' => 'text',
            'name' => 'repeat_from',
            'label'=>'Repetir desde',
            'class'=>'_repeat',
            'required'=> false
        ));
        
        $this->addElement(array(
            'type' => 'text',
            'name' => 'repeat_to',
            'label'=>'Repetir hasta',
            'class'=>'_repeat',
            'required'=> false
        ));
        
        $this->addElement(array(
            'type' => 'text',
            'name' => 'repeat_hour',
            'class'=>'_repeat',
            'label'=>'Hora de envio',
            'required'=> false
        ));
        
        $this->addElement(array(
            'type' => 'select',
            'name' => 'status',
            'label'=>'Status',
            'multiOptions' => $this->getListStatus(),
            'required'=> true
        ));
        
        $this->addElement(array(
            'type' => 'button',
            'name' => 'send',
            'value'=> $this->_getTranslation('Terminar'),
            'optionals'=>array(
                'onclick'=>"submit()"),
            'class'=>'btn btn-primary m-t-1 m-b-1'
        ));
        
        $this->addElement(array(
            'type' => 'button',
            'name' => 'cancelar',
            'value'=> $this->_getTranslation('Cancelar'),
            'optionals'=>array(
            'onclick'=>"document.location = '".ROOT_HOST."/Controller/Notification.php'"),
            'class'=>'btn btn-danger m-t-1 m-b-1'
        ));
    }
    
    public function getListStatus(){
        $repository = new NotificationRepository();
        $list = $repository->getListStatus();
         
        $array = array();
        foreach($list as $key => $value){
            $array[$key] = $value;
        }
        return $array;        
    }
    
    public function getListUsers(){
        $repository = new UsuarioRepository();
        $list = $repository->getListaSelectUsuarios();
         
        $array = array('0'=>'Todos'); 
        foreach($list as $key => $value){
            $array[$key] = $value;
        }
        return $array;
    }
    
    public function populate($data){       
        $tools = new Tools();       
        if(isset($data['when_send']) && $tools->isValidaDateYYYMMDD($data['when_send'])){
            $data['when_send'] = $tools->setFormatDateTimeToForm($data['when_send']);
        }
        
        if(isset($data['repeat_from']) && $tools->isValidaDateYYYMMDD($data['repeat_from'])){
            $data['repeat_from'] = $tools->setFormatDateToForm($data['repeat_from']);
        }
        
        if(isset($data['repeat_to']) && $tools->isValidaDateYYYMMDD($data['repeat_to'])){
            $data['repeat_to'] = $tools->setFormatDateToForm($data['repeat_to']);
        }
        
        if(isset($data['send_now']) && $data['send_now'] == '1'){
            $this->addOptionals('send_now', array('checked'=>true));
            $this->setValueToElement('1', 'send_now');
        }        
        
        if(isset($data['repeat']) && $data['repeat'] == '1'){
            $this->addOptionals('repeat', array('checked'=>true));
            $this->setValueToElement('1', 'repeat');
        }        
      
        if(isset($data['send_to']) && $data['send_to']!=null && is_array($data['send_to'])){
            $sendTo = array();
            foreach($data['send_to'] as $user){
                $sendTo[$user] = $user;
            }
       
            $data['send_to'] = $sendTo;
        }
        
        parent::populate($data);
    }
    
    public function isValid() {
        if(isset($_POST['repeat'])){
            $this->setAsRequired(array('repeat_from','repeat_to','repeat_hour'));
        }
        
        if(isset($_POST['send_now'])){
            $this->noRequired(array('when_send'));
        }
        
        $valid = parent::isValid();               
        
        $flashmessenger = new FlashMessenger();
        if($valid){
            $message = $this->getValueElement('message');
             if(is_null($message) || empty($message)){
                $flashmessenger->addMessage(array('danger'=>'El campo <strong>Mensaje</strong> es requerido.'));
                return null;
            }    
            
            $sendTo = $this->getValueElement('send_to');
            if(is_null($sendTo)){
                $flashmessenger->addMessage(array('danger'=>'El campo <strong>Para</strong> es requerido.'));
                return null;
            }            
        }
         
        return $valid;
    }
}